from typing import Any, Awaitable, Callable, Protocol

from kio.schema.errors import ErrorCode

from icestream.kafkaserver.handlers.add_offsets_to_txn import AddOffsetsToTxnRequestHeader, AddOffsetsToTxnRequest, \
    AddOffsetsToTxnResponse
from icestream.kafkaserver.handlers.add_partitions_to_txn import AddPartitionsToTxnRequestHeader, \
    AddPartitionsToTxnRequest, AddPartitionsToTxnResponse
from icestream.kafkaserver.handlers.add_raft_voter import AddRaftVoterRequestHeader, AddRaftVoterRequest, \
    AddRaftVoterResponse
from icestream.kafkaserver.handlers.alter_client_quotas import AlterClientQuotasRequestHeader, AlterClientQuotasRequest, \
    AlterClientQuotasResponse
from icestream.kafkaserver.handlers.api_versions import ApiVersionsRequestHeader, ApiVersionsRequest, \
    ApiVersionsResponse
from icestream.kafkaserver.handlers.create_topics import CreateTopicsRequest, CreateTopicsResponse, \
    CreateTopicsRequestHeader
from icestream.kafkaserver.handlers.delete_topics import DeleteTopicsRequestHeader, DeleteTopicsRequest, \
    DeleteTopicsResponse
from icestream.kafkaserver.handlers.delete_acls import (
    DeleteAclsRequest,
    DeleteAclsRequestHeader,
    DeleteAclsResponse,
)
from icestream.kafkaserver.handlers.delete_groups import (
    DeleteGroupsRequest,
    DeleteGroupsRequestHeader,
    DeleteGroupsResponse,
)
from icestream.kafkaserver.handlers.delete_records import (
    DeleteRecordsRequest,
    DeleteRecordsRequestHeader,
    DeleteRecordsResponse,
)
from icestream.kafkaserver.handlers.delete_share_group_state import (
    DeleteShareGroupStateRequest,
    DeleteShareGroupStateRequestHeader,
    DeleteShareGroupStateResponse,
)
from icestream.kafkaserver.handlers.describe_acls import (
    DescribeAclsRequest,
    DescribeAclsRequestHeader,
    DescribeAclsResponse,
)
from icestream.kafkaserver.handlers.describe_client_quotas import (
    DescribeClientQuotasRequest,
    DescribeClientQuotasRequestHeader,
    DescribeClientQuotasResponse,
)
from icestream.kafkaserver.handlers.describe_cluster import (
    DescribeClusterRequest,
    DescribeClusterRequestHeader,
    DescribeClusterResponse,
)
from icestream.kafkaserver.handlers.describe_configs import (
    DescribeConfigsRequest,
    DescribeConfigsRequestHeader,
    DescribeConfigsResponse,
)
from icestream.kafkaserver.handlers.describe_delegation_token import (
    DescribeDelegationTokenRequest,
    DescribeDelegationTokenRequestHeader,
    DescribeDelegationTokenResponse,
)
from icestream.kafkaserver.handlers.describe_groups import (
    DescribeGroupsRequest,
    DescribeGroupsRequestHeader,
    DescribeGroupsResponse,
)
from icestream.kafkaserver.handlers.describe_log_dirs import (
    DescribeLogDirsRequest,
    DescribeLogDirsRequestHeader,
    DescribeLogDirsResponse,
)
from icestream.kafkaserver.handlers.describe_producers import (
    DescribeProducersRequest,
    DescribeProducersRequestHeader,
    DescribeProducersResponse,
)
from icestream.kafkaserver.handlers.describe_quorum import (
    DescribeQuorumRequest,
    DescribeQuorumRequestHeader,
    DescribeQuorumResponse,
)
from icestream.kafkaserver.handlers.describe_topic_partitions import (
    DescribeTopicPartitionsRequest,
    DescribeTopicPartitionsRequestHeader,
    DescribeTopicPartitionsResponse,
)
from icestream.kafkaserver.handlers.describe_transactions import (
    DescribeTransactionsRequest,
    DescribeTransactionsRequestHeader,
    DescribeTransactionsResponse,
)
from icestream.kafkaserver.handlers.describe_user_scram_credentials import (
    DescribeUserScramCredentialsRequest,
    DescribeUserScramCredentialsRequestHeader,
    DescribeUserScramCredentialsResponse,
)
from icestream.kafkaserver.handlers.elect_leaders import (
    ElectLeadersRequest,
    ElectLeadersRequestHeader,
    ElectLeadersResponse,
)
from icestream.kafkaserver.handlers.end_quorum_epoch import (
    EndQuorumEpochRequest,
    EndQuorumEpochRequestHeader,
    EndQuorumEpochResponse,
)
from icestream.kafkaserver.handlers.end_txn import (
    EndTxnRequest,
    EndTxnRequestHeader,
    EndTxnResponse,
)
from icestream.kafkaserver.handlers.envelope import (
    EnvelopeRequest,
    EnvelopeRequestHeader,
    EnvelopeResponse,
)
from icestream.kafkaserver.handlers.create_acls import (
    CreateAclsRequest,
    CreateAclsRequestHeader,
    CreateAclsResponse,
)
from icestream.kafkaserver.handlers.create_delegation_token import (
    CreateDelegationTokenRequest,
    CreateDelegationTokenRequestHeader,
    CreateDelegationTokenResponse,
)
from icestream.kafkaserver.handlers.create_partitions import (
    CreatePartitionsRequest,
    CreatePartitionsRequestHeader,
    CreatePartitionsResponse,
)
from icestream.kafkaserver.handlers.metadata import (
    MetadataRequest,
    MetadataRequestHeader,
    MetadataResponse,
)
from icestream.kafkaserver.handlers.produce import (
    ProduceRequest,
    ProduceRequestHeader,
    ProduceResponse,
)

from icestream.kafkaserver.handlers.fetch import (
    FetchRequest,
    FetchRequestHeader,
    FetchResponse,
)
from icestream.kafkaserver.handlers.allocate_producer_ids import AllocateProducerIdsRequestHeader, \
    AllocateProducerIdsRequest, AllocateProducerIdsResponse

from icestream.kafkaserver.handlers.alter_configs import (
    AlterConfigsRequest,
    AlterConfigsRequestHeader,
    AlterConfigsResponse,
)
from icestream.kafkaserver.handlers.alter_partition import (
    AlterPartitionRequest,
    AlterPartitionRequestHeader,
    AlterPartitionResponse,
)
from icestream.kafkaserver.handlers.alter_partition_reassignments import (
    AlterPartitionReassignmentsRequest,
    AlterPartitionReassignmentsRequestHeader,
    AlterPartitionReassignmentsResponse,
)
from icestream.kafkaserver.handlers.alter_replica_log_dirs import (
    AlterReplicaLogDirsRequest,
    AlterReplicaLogDirsRequestHeader,
    AlterReplicaLogDirsResponse,
)
from icestream.kafkaserver.handlers.alter_user_scram_credentials import (
    AlterUserScramCredentialsRequest,
    AlterUserScramCredentialsRequestHeader,
    AlterUserScramCredentialsResponse,
)
from icestream.kafkaserver.handlers.assign_replicas_to_dirs import (
    AssignReplicasToDirsRequest,
    AssignReplicasToDirsRequestHeader,
    AssignReplicasToDirsResponse,
)
from icestream.kafkaserver.handlers.begin_quorum_epoch import (
    BeginQuorumEpochRequest,
    BeginQuorumEpochRequestHeader,
    BeginQuorumEpochResponse,
)
from icestream.kafkaserver.handlers.broker_heartbeat import (
    BrokerHeartbeatRequest,
    BrokerHeartbeatRequestHeader,
    BrokerHeartbeatResponse,
)
from icestream.kafkaserver.handlers.broker_registration import (
    BrokerRegistrationRequest,
    BrokerRegistrationRequestHeader,
    BrokerRegistrationResponse,
)
from icestream.kafkaserver.handlers.consumer_group_describe import (
    ConsumerGroupDescribeRequest,
    ConsumerGroupDescribeRequestHeader,
    ConsumerGroupDescribeResponse,
)
from icestream.kafkaserver.handlers.consumer_group_heartbeat import (
    ConsumerGroupHeartbeatRequest,
    ConsumerGroupHeartbeatRequestHeader,
    ConsumerGroupHeartbeatResponse,
)
from icestream.kafkaserver.handlers.controlled_shutdown import (
    ControlledShutdownRequest,
    ControlledShutdownRequestHeader,
    ControlledShutdownResponse,
)
from icestream.kafkaserver.handlers.controller_registration import (
    ControllerRegistrationRequest,
    ControllerRegistrationRequestHeader,
    ControllerRegistrationResponse,
)
from icestream.kafkaserver.handlers.find_coordinator import (
    FindCoordinatorRequest,
    FindCoordinatorRequestHeader,
    FindCoordinatorResponse,
)
from icestream.kafkaserver.handlers.expire_delegation_token import (
    ExpireDelegationTokenRequest,
    ExpireDelegationTokenRequestHeader,
    ExpireDelegationTokenResponse,
)
from icestream.kafkaserver.handlers.fetch_snapshot import (
    FetchSnapshotRequest,
    FetchSnapshotRequestHeader,
    FetchSnapshotResponse,
)
from icestream.kafkaserver.handlers.get_telemetry_subscriptions import (
    GetTelemetrySubscriptionsRequest,
    GetTelemetrySubscriptionsRequestHeader,
    GetTelemetrySubscriptionsResponse,
)
from icestream.kafkaserver.handlers.heartbeat import (
    HeartbeatRequest,
    HeartbeatRequestHeader,
    HeartbeatResponse,
)
from icestream.kafkaserver.handlers.incremental_alter_configs import (
    IncrementalAlterConfigsRequest,
    IncrementalAlterConfigsRequestHeader,
    IncrementalAlterConfigsResponse,
)
from icestream.kafkaserver.handlers.init_producer_id import (
    InitProducerIdRequest,
    InitProducerIdRequestHeader,
    InitProducerIdResponse,
)
from icestream.kafkaserver.handlers.initialize_share_group_state import (
    InitializeShareGroupStateRequest,
    InitializeShareGroupStateRequestHeader,
    InitializeShareGroupStateResponse,
)
from icestream.kafkaserver.handlers.join_group import (
    JoinGroupRequest,
    JoinGroupRequestHeader,
    JoinGroupResponse,
)
from icestream.kafkaserver.handlers.leader_and_isr import (
    LeaderAndIsrRequest,
    LeaderAndIsrRequestHeader,
    LeaderAndIsrResponse,
)
from icestream.kafkaserver.handlers.leave_group import (
    LeaveGroupRequest,
    LeaveGroupRequestHeader,
    LeaveGroupResponse,
)
from icestream.kafkaserver.handlers.list_client_metrics_resources import (
    ListClientMetricsResourcesRequest,
    ListClientMetricsResourcesRequestHeader,
    ListClientMetricsResourcesResponse,
)
from icestream.kafkaserver.handlers.list_groups import (
    ListGroupsRequest,
    ListGroupsRequestHeader,
    ListGroupsResponse,
)
from icestream.kafkaserver.handlers.list_offsets import (
    ListOffsetsRequest,
    ListOffsetsRequestHeader,
    ListOffsetsResponse,
)
from icestream.kafkaserver.handlers.list_partition_reassignments import (
    ListPartitionReassignmentsRequest,
    ListPartitionReassignmentsRequestHeader,
    ListPartitionReassignmentsResponse,
)
from icestream.kafkaserver.handlers.list_transactions import (
    ListTransactionsRequest,
    ListTransactionsRequestHeader,
    ListTransactionsResponse,
)
from icestream.kafkaserver.handlers.offset_commit import (
    OffsetCommitRequest,
    OffsetCommitRequestHeader,
    OffsetCommitResponse,
)
from icestream.kafkaserver.handlers.offset_delete import (
    OffsetDeleteRequest,
    OffsetDeleteRequestHeader,
    OffsetDeleteResponse,
)
from icestream.kafkaserver.handlers.offset_fetch import (
    OffsetFetchRequest,
    OffsetFetchRequestHeader,
    OffsetFetchResponse,
)
from icestream.kafkaserver.handlers.offset_for_leader_epoch import (
    OffsetForLeaderEpochRequest,
    OffsetForLeaderEpochRequestHeader,
    OffsetForLeaderEpochResponse,
)
from icestream.kafkaserver.handlers.push_telemetry import (
    PushTelemetryRequest,
    PushTelemetryRequestHeader,
    PushTelemetryResponse,
)
from icestream.kafkaserver.handlers.read_share_group_state import (
    ReadShareGroupStateRequest,
    ReadShareGroupStateRequestHeader,
    ReadShareGroupStateResponse,
)
from icestream.kafkaserver.handlers.read_share_group_state_summary import (
    ReadShareGroupStateSummaryRequest,
    ReadShareGroupStateSummaryRequestHeader,
    ReadShareGroupStateSummaryResponse,
)
from icestream.kafkaserver.handlers.remove_raft_voter import (
    RemoveRaftVoterRequest,
    RemoveRaftVoterRequestHeader,
    RemoveRaftVoterResponse,
)
from icestream.kafkaserver.handlers.renew_delegation_token import (
    RenewDelegationTokenRequest,
    RenewDelegationTokenRequestHeader,
    RenewDelegationTokenResponse,
)
from icestream.kafkaserver.handlers.sasl_authenticate import (
    SaslAuthenticateRequest,
    SaslAuthenticateRequestHeader,
    SaslAuthenticateResponse,
)
from icestream.kafkaserver.handlers.sasl_handshake import (
    SaslHandshakeRequest,
    SaslHandshakeRequestHeader,
    SaslHandshakeResponse,
)
from icestream.kafkaserver.handlers.share_acknowledge import (
    ShareAcknowledgeRequest,
    ShareAcknowledgeRequestHeader,
    ShareAcknowledgeResponse,
)
from icestream.kafkaserver.handlers.share_fetch import (
    ShareFetchRequest,
    ShareFetchRequestHeader,
    ShareFetchResponse,
)
from icestream.kafkaserver.handlers.share_group_describe import (
    ShareGroupDescribeRequest,
    ShareGroupDescribeRequestHeader,
    ShareGroupDescribeResponse,
)
from icestream.kafkaserver.handlers.share_group_heartbeat import (
    ShareGroupHeartbeatRequest,
    ShareGroupHeartbeatRequestHeader,
    ShareGroupHeartbeatResponse,
)
from icestream.kafkaserver.handlers.stop_replica import (
    StopReplicaRequest,
    StopReplicaRequestHeader,
    StopReplicaResponse,
)
from icestream.kafkaserver.handlers.sync_group import (
    SyncGroupRequest,
    SyncGroupRequestHeader,
    SyncGroupResponse,
)
from icestream.kafkaserver.handlers.txn_offset_commit import (
    TxnOffsetCommitRequest,
    TxnOffsetCommitRequestHeader,
    TxnOffsetCommitResponse,
)
from icestream.kafkaserver.handlers.unregister_broker import (
    UnregisterBrokerRequest,
    UnregisterBrokerRequestHeader,
    UnregisterBrokerResponse,
)
from icestream.kafkaserver.handlers.update_features import (
    UpdateFeaturesRequest,
    UpdateFeaturesRequestHeader,
    UpdateFeaturesResponse,
)
from icestream.kafkaserver.handlers.update_metadata import (
    UpdateMetadataRequest,
    UpdateMetadataRequestHeader,
    UpdateMetadataResponse,
)
from icestream.kafkaserver.handlers.vote import (
    VoteRequest,
    VoteRequestHeader,
    VoteResponse,
)
from icestream.kafkaserver.handlers.write_txn_markers import (
    WriteTxnMarkersRequest,
    WriteTxnMarkersRequestHeader,
    WriteTxnMarkersResponse,
)
from icestream.kafkaserver.handlers.update_raft_voter import (
    UpdateRaftVoterRequest,
    UpdateRaftVoterRequestHeader,
    UpdateRaftVoterResponse,
)
from icestream.kafkaserver.handlers.write_share_group_state import (
    WriteShareGroupStateRequest,
    WriteShareGroupStateRequestHeader,
    WriteShareGroupStateResponse,
)


class KafkaHandler(Protocol):
    async def handle_write_share_group_state_request(
        self,
        header: WriteShareGroupStateRequestHeader,
        req: WriteShareGroupStateRequest,
        api_version: int,
        callback: Callable[[WriteShareGroupStateResponse], Awaitable[None]],
    ):
        pass

    def write_share_group_state_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: WriteShareGroupStateRequest,
        api_version: int,
    ) -> WriteShareGroupStateResponse:
        pass

    async def handle_update_raft_voter_request(
        self,
        header: UpdateRaftVoterRequestHeader,
        req: UpdateRaftVoterRequest,
        api_version: int,
        callback: Callable[[UpdateRaftVoterResponse], Awaitable[None]],
    ):
        pass

    def update_raft_voter_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: UpdateRaftVoterRequest,
        api_version: int,
    ) -> UpdateRaftVoterResponse:
        pass

    async def handle_write_txn_markers_request(
        self,
        header: WriteTxnMarkersRequestHeader,
        req: WriteTxnMarkersRequest,
        api_version: int,
        callback: Callable[[WriteTxnMarkersResponse], Awaitable[None]],
    ):
        pass

    def write_txn_markers_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: WriteTxnMarkersRequest,
        api_version: int,
    ) -> WriteTxnMarkersResponse:
        pass

    async def handle_vote_request(
        self,
        header: VoteRequestHeader,
        req: VoteRequest,
        api_version: int,
        callback: Callable[[VoteResponse], Awaitable[None]],
    ):
        pass

    def vote_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: VoteRequest,
        api_version: int,
    ) -> VoteResponse:
        pass

    async def handle_update_metadata_request(
        self,
        header: UpdateMetadataRequestHeader,
        req: UpdateMetadataRequest,
        api_version: int,
        callback: Callable[[UpdateMetadataResponse], Awaitable[None]],
    ):
        pass

    def update_metadata_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: UpdateMetadataRequest,
        api_version: int,
    ) -> UpdateMetadataResponse:
        pass

    async def handle_update_features_request(
        self,
        header: UpdateFeaturesRequestHeader,
        req: UpdateFeaturesRequest,
        api_version: int,
        callback: Callable[[UpdateFeaturesResponse], Awaitable[None]],
    ):
        pass

    def update_features_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: UpdateFeaturesRequest,
        api_version: int,
    ) -> UpdateFeaturesResponse:
        pass

    async def handle_unregister_broker_request(
        self,
        header: UnregisterBrokerRequestHeader,
        req: UnregisterBrokerRequest,
        api_version: int,
        callback: Callable[[UnregisterBrokerResponse], Awaitable[None]],
    ):
        pass

    def unregister_broker_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: UnregisterBrokerRequest,
        api_version: int,
    ) -> UnregisterBrokerResponse:
        pass

    async def handle_txn_offset_commit_request(
        self,
        header: TxnOffsetCommitRequestHeader,
        req: TxnOffsetCommitRequest,
        api_version: int,
        callback: Callable[[TxnOffsetCommitResponse], Awaitable[None]],
    ):
        pass

    def txn_offset_commit_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: TxnOffsetCommitRequest,
        api_version: int,
    ) -> TxnOffsetCommitResponse:
        pass

    async def handle_sync_group_request(
        self,
        header: SyncGroupRequestHeader,
        req: SyncGroupRequest,
        api_version: int,
        callback: Callable[[SyncGroupResponse], Awaitable[None]],
    ):
        pass

    def sync_group_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: SyncGroupRequest,
        api_version: int,
    ) -> SyncGroupResponse:
        pass

    async def handle_stop_replica_request(
        self,
        header: StopReplicaRequestHeader,
        req: StopReplicaRequest,
        api_version: int,
        callback: Callable[[StopReplicaResponse], Awaitable[None]],
    ):
        pass

    def stop_replica_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: StopReplicaRequest,
        api_version: int,
    ) -> StopReplicaResponse:
        pass

    async def handle_share_group_heartbeat_request(
        self,
        header: ShareGroupHeartbeatRequestHeader,
        req: ShareGroupHeartbeatRequest,
        api_version: int,
        callback: Callable[[ShareGroupHeartbeatResponse], Awaitable[None]],
    ):
        pass

    def share_group_heartbeat_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: ShareGroupHeartbeatRequest,
        api_version: int,
    ) -> ShareGroupHeartbeatResponse:
        pass

    async def handle_share_group_describe_request(
        self,
        header: ShareGroupDescribeRequestHeader,
        req: ShareGroupDescribeRequest,
        api_version: int,
        callback: Callable[[ShareGroupDescribeResponse], Awaitable[None]],
    ):
        pass

    def share_group_describe_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: ShareGroupDescribeRequest,
        api_version: int,
    ) -> ShareGroupDescribeResponse:
        pass

    async def handle_share_fetch_request(
        self,
        header: ShareFetchRequestHeader,
        req: ShareFetchRequest,
        api_version: int,
        callback: Callable[[ShareFetchResponse], Awaitable[None]],
    ):
        pass

    def share_fetch_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: ShareFetchRequest,
        api_version: int,
    ) -> ShareFetchResponse:
        pass

    async def handle_share_acknowledge_request(
        self,
        header: ShareAcknowledgeRequestHeader,
        req: ShareAcknowledgeRequest,
        api_version: int,
        callback: Callable[[ShareAcknowledgeResponse], Awaitable[None]],
    ):
        pass

    def share_acknowledge_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: ShareAcknowledgeRequest,
        api_version: int,
    ) -> ShareAcknowledgeResponse:
        pass

    async def handle_sasl_handshake_request(
        self,
        header: SaslHandshakeRequestHeader,
        req: SaslHandshakeRequest,
        api_version: int,
        callback: Callable[[SaslHandshakeResponse], Awaitable[None]],
    ):
        pass

    def sasl_handshake_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: SaslHandshakeRequest,
        api_version: int,
    ) -> SaslHandshakeResponse:
        pass

    async def handle_sasl_authenticate_request(
        self,
        header: SaslAuthenticateRequestHeader,
        req: SaslAuthenticateRequest,
        api_version: int,
        callback: Callable[[SaslAuthenticateResponse], Awaitable[None]],
    ):
        pass

    def sasl_authenticate_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: SaslAuthenticateRequest,
        api_version: int,
    ) -> SaslAuthenticateResponse:
        pass

    async def handle_renew_delegation_token_request(
        self,
        header: RenewDelegationTokenRequestHeader,
        req: RenewDelegationTokenRequest,
        api_version: int,
        callback: Callable[[RenewDelegationTokenResponse], Awaitable[None]],
    ):
        pass

    def renew_delegation_token_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: RenewDelegationTokenRequest,
        api_version: int,
    ) -> RenewDelegationTokenResponse:
        pass

    async def handle_remove_raft_voter_request(
        self,
        header: RemoveRaftVoterRequestHeader,
        req: RemoveRaftVoterRequest,
        api_version: int,
        callback: Callable[[RemoveRaftVoterResponse], Awaitable[None]],
    ):
        pass

    def remove_raft_voter_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: RemoveRaftVoterRequest,
        api_version: int,
    ) -> RemoveRaftVoterResponse:
        pass

    async def handle_read_share_group_state_summary_request(
        self,
        header: ReadShareGroupStateSummaryRequestHeader,
        req: ReadShareGroupStateSummaryRequest,
        api_version: int,
        callback: Callable[[ReadShareGroupStateSummaryResponse], Awaitable[None]],
    ):
        pass

    def read_share_group_state_summary_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: ReadShareGroupStateSummaryRequest,
        api_version: int,
    ) -> ReadShareGroupStateSummaryResponse:
        pass

    async def handle_read_share_group_state_request(
        self,
        header: ReadShareGroupStateRequestHeader,
        req: ReadShareGroupStateRequest,
        api_version: int,
        callback: Callable[[ReadShareGroupStateResponse], Awaitable[None]],
    ):
        pass

    def read_share_group_state_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: ReadShareGroupStateRequest,
        api_version: int,
    ) -> ReadShareGroupStateResponse:
        pass

    async def handle_push_telemetry_request(
        self,
        header: PushTelemetryRequestHeader,
        req: PushTelemetryRequest,
        api_version: int,
        callback: Callable[[PushTelemetryResponse], Awaitable[None]],
    ):
        pass

    def push_telemetry_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: PushTelemetryRequest,
        api_version: int,
    ) -> PushTelemetryResponse:
        pass

    async def handle_offset_for_leader_epoch_request(
        self,
        header: OffsetForLeaderEpochRequestHeader,
        req: OffsetForLeaderEpochRequest,
        api_version: int,
        callback: Callable[[OffsetForLeaderEpochResponse], Awaitable[None]],
    ):
        pass

    def offset_for_leader_epoch_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: OffsetForLeaderEpochRequest,
        api_version: int,
    ) -> OffsetForLeaderEpochResponse:
        pass

    async def handle_offset_fetch_request(
        self,
        header: OffsetFetchRequestHeader,
        req: OffsetFetchRequest,
        api_version: int,
        callback: Callable[[OffsetFetchResponse], Awaitable[None]],
    ):
        pass

    def offset_fetch_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: OffsetFetchRequest,
        api_version: int,
    ) -> OffsetFetchResponse:
        pass

    async def handle_offset_delete_request(
        self,
        header: OffsetDeleteRequestHeader,
        req: OffsetDeleteRequest,
        api_version: int,
        callback: Callable[[OffsetDeleteResponse], Awaitable[None]],
    ):
        pass

    def offset_delete_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: OffsetDeleteRequest,
        api_version: int,
    ) -> OffsetDeleteResponse:
        pass

    async def handle_offset_commit_request(
        self,
        header: OffsetCommitRequestHeader,
        req: OffsetCommitRequest,
        api_version: int,
        callback: Callable[[OffsetCommitResponse], Awaitable[None]],
    ):
        pass

    def offset_commit_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: OffsetCommitRequest,
        api_version: int,
    ) -> OffsetCommitResponse:
        pass

    async def handle_list_transactions_request(
        self,
        header: ListTransactionsRequestHeader,
        req: ListTransactionsRequest,
        api_version: int,
        callback: Callable[[ListTransactionsResponse], Awaitable[None]],
    ):
        pass

    def list_transactions_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: ListTransactionsRequest,
        api_version: int,
    ) -> ListTransactionsResponse:
        pass

    async def handle_list_partition_reassignments_request(
        self,
        header: ListPartitionReassignmentsRequestHeader,
        req: ListPartitionReassignmentsRequest,
        api_version: int,
        callback: Callable[[ListPartitionReassignmentsResponse], Awaitable[None]],
    ):
        pass

    def list_partition_reassignments_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: ListPartitionReassignmentsRequest,
        api_version: int,
    ) -> ListPartitionReassignmentsResponse:
        pass

    async def handle_list_offsets_request(
        self,
        header: ListOffsetsRequestHeader,
        req: ListOffsetsRequest,
        api_version: int,
        callback: Callable[[ListOffsetsResponse], Awaitable[None]],
    ):
        pass

    def list_offsets_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: ListOffsetsRequest,
        api_version: int,
    ) -> ListOffsetsResponse:
        pass

    async def handle_list_groups_request(
        self,
        header: ListGroupsRequestHeader,
        req: ListGroupsRequest,
        api_version: int,
        callback: Callable[[ListGroupsResponse], Awaitable[None]],
    ):
        pass

    def list_groups_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: ListGroupsRequest,
        api_version: int,
    ) -> ListGroupsResponse:
        pass

    async def handle_list_client_metrics_resources_request(
        self,
        header: ListClientMetricsResourcesRequestHeader,
        req: ListClientMetricsResourcesRequest,
        api_version: int,
        callback: Callable[[ListClientMetricsResourcesResponse], Awaitable[None]],
    ):
        pass

    def list_client_metrics_resources_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: ListClientMetricsResourcesRequest,
        api_version: int,
    ) -> ListClientMetricsResourcesResponse:
        pass

    async def handle_leave_group_request(
        self,
        header: LeaveGroupRequestHeader,
        req: LeaveGroupRequest,
        api_version: int,
        callback: Callable[[LeaveGroupResponse], Awaitable[None]],
    ):
        pass

    def leave_group_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: LeaveGroupRequest,
        api_version: int,
    ) -> LeaveGroupResponse:
        pass

    async def handle_leader_and_isr_request(
        self,
        header: LeaderAndIsrRequestHeader,
        req: LeaderAndIsrRequest,
        api_version: int,
        callback: Callable[[LeaderAndIsrResponse], Awaitable[None]],
    ):
        pass

    def leader_and_isr_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: LeaderAndIsrRequest,
        api_version: int,
    ) -> LeaderAndIsrResponse:
        pass

    async def handle_join_group_request(
        self,
        header: JoinGroupRequestHeader,
        req: JoinGroupRequest,
        api_version: int,
        callback: Callable[[JoinGroupResponse], Awaitable[None]],
    ):
        pass

    def join_group_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: JoinGroupRequest,
        api_version: int,
    ) -> JoinGroupResponse:
        pass

    async def handle_initialize_share_group_state_request(
        self,
        header: InitializeShareGroupStateRequestHeader,
        req: InitializeShareGroupStateRequest,
        api_version: int,
        callback: Callable[[InitializeShareGroupStateResponse], Awaitable[None]],
    ):
        pass

    def initialize_share_group_state_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: InitializeShareGroupStateRequest,
        api_version: int,
    ) -> InitializeShareGroupStateResponse:
        pass

    async def handle_init_producer_id_request(
        self,
        header: InitProducerIdRequestHeader,
        req: InitProducerIdRequest,
        api_version: int,
        callback: Callable[[InitProducerIdResponse], Awaitable[None]],
    ):
        pass

    def init_producer_id_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: InitProducerIdRequest,
        api_version: int,
    ) -> InitProducerIdResponse:
        pass

    async def handle_incremental_alter_configs_request(
        self,
        header: IncrementalAlterConfigsRequestHeader,
        req: IncrementalAlterConfigsRequest,
        api_version: int,
        callback: Callable[[IncrementalAlterConfigsResponse], Awaitable[None]],
    ):
        pass

    def incremental_alter_configs_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: IncrementalAlterConfigsRequest,
        api_version: int,
    ) -> IncrementalAlterConfigsResponse:
        pass

    async def handle_heartbeat_request(
        self,
        header: HeartbeatRequestHeader,
        req: HeartbeatRequest,
        api_version: int,
        callback: Callable[[HeartbeatResponse], Awaitable[None]],
    ):
        pass

    def heartbeat_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: HeartbeatRequest,
        api_version: int,
    ) -> HeartbeatResponse:
        pass

    async def handle_get_telemetry_subscriptions_request(
        self,
        header: GetTelemetrySubscriptionsRequestHeader,
        req: GetTelemetrySubscriptionsRequest,
        api_version: int,
        callback: Callable[[GetTelemetrySubscriptionsResponse], Awaitable[None]],
    ):
        pass

    def get_telemetry_subscriptions_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: GetTelemetrySubscriptionsRequest,
        api_version: int,
    ) -> GetTelemetrySubscriptionsResponse:
        pass

    async def handle_fetch_snapshot_request(
        self,
        header: FetchSnapshotRequestHeader,
        req: FetchSnapshotRequest,
        api_version: int,
        callback: Callable[[FetchSnapshotResponse], Awaitable[None]],
    ):
        pass

    def fetch_snapshot_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: FetchSnapshotRequest,
        api_version: int,
    ) -> FetchSnapshotResponse:
        pass

    async def handle_expire_delegation_token_request(
        self,
        header: ExpireDelegationTokenRequestHeader,
        req: ExpireDelegationTokenRequest,
        api_version: int,
        callback: Callable[[ExpireDelegationTokenResponse], Awaitable[None]],
    ):
        pass

    def expire_delegation_token_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: ExpireDelegationTokenRequest,
        api_version: int,
    ) -> ExpireDelegationTokenResponse:
        pass

    async def handle_find_coordinator_request(
        self,
        header: FindCoordinatorRequestHeader,
        req: FindCoordinatorRequest,
        api_version: int,
        callback: Callable[[FindCoordinatorResponse], Awaitable[None]],
    ):
        pass

    def find_coordinator_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: FindCoordinatorRequest,
        api_version: int,
    ) -> FindCoordinatorResponse:
        pass

    async def handle_produce_request(
        self,
        header: ProduceRequestHeader,
        req: ProduceRequest,
        api_version: int,
        callback: Callable[[ProduceResponse], Any],
    ):
        pass

    def produce_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: ProduceRequest,
        api_version: int,
    ) -> ProduceResponse:
        pass

    async def handle_metadata_request(
        self,
        header: MetadataRequestHeader,
        req: MetadataRequest,
        api_version: int,
        callback: Callable[[MetadataResponse], Any],
    ):
        pass

    def metadata_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: MetadataRequest,
        api_version: int,
    ) -> MetadataResponse:
        pass

    async def handle_api_versions_request(
        self,
        header: ApiVersionsRequestHeader,
        req: ApiVersionsRequest,
        api_version: int,
        callback: Callable[[ApiVersionsResponse], Awaitable[None]],
    ):
        pass

    def api_versions_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: ApiVersionsRequest,
        api_version: int,
    ) -> ApiVersionsResponse:
        pass

    async def handle_create_topics_request(
        self,
        header: CreateTopicsRequestHeader,
        req: CreateTopicsRequest,
        api_version: int,
        callback: Callable[[CreateTopicsResponse], Awaitable[None]],
    ):
        pass

    def create_topics_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: CreateTopicsRequest,
        api_version: int,
    ) -> CreateTopicsResponse:
        pass

    async def handle_fetch_request(
        self,
        header: FetchRequestHeader,
        req: FetchRequest,
        api_version: int,
        callback: Callable[[FetchResponse], Awaitable[None]],
    ) -> FetchResponse:
        pass

    def fetch_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: FetchRequest,
        api_version: int,
    ) -> FetchResponse:
        pass

    async def handle_delete_topics_request(
            self,
            header: DeleteTopicsRequestHeader,
            req: DeleteTopicsRequest,
            api_version: int,
            callback: Callable[[DeleteTopicsResponse], Awaitable[None]],
    ):
        pass

    def delete_topics_request_error_response(
            self,
            error_code: ErrorCode,
            error_message: str,
            req: DeleteTopicsRequest,
            api_version: int,
    ) -> DeleteTopicsResponse:
        pass

    async def handle_delete_acls_request(
        self,
        header: DeleteAclsRequestHeader,
        req: DeleteAclsRequest,
        api_version: int,
        callback: Callable[[DeleteAclsResponse], Awaitable[None]],
    ):
        pass

    def delete_acls_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: DeleteAclsRequest,
        api_version: int,
    ) -> DeleteAclsResponse:
        pass

    async def handle_delete_groups_request(
        self,
        header: DeleteGroupsRequestHeader,
        req: DeleteGroupsRequest,
        api_version: int,
        callback: Callable[[DeleteGroupsResponse], Awaitable[None]],
    ):
        pass

    def delete_groups_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: DeleteGroupsRequest,
        api_version: int,
    ) -> DeleteGroupsResponse:
        pass

    async def handle_delete_records_request(
        self,
        header: DeleteRecordsRequestHeader,
        req: DeleteRecordsRequest,
        api_version: int,
        callback: Callable[[DeleteRecordsResponse], Awaitable[None]],
    ):
        pass

    def delete_records_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: DeleteRecordsRequest,
        api_version: int,
    ) -> DeleteRecordsResponse:
        pass

    async def handle_delete_share_group_state_request(
        self,
        header: DeleteShareGroupStateRequestHeader,
        req: DeleteShareGroupStateRequest,
        api_version: int,
        callback: Callable[[DeleteShareGroupStateResponse], Awaitable[None]],
    ):
        pass

    def delete_share_group_state_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: DeleteShareGroupStateRequest,
        api_version: int,
    ) -> DeleteShareGroupStateResponse:
        pass

    async def handle_describe_acls_request(
        self,
        header: DescribeAclsRequestHeader,
        req: DescribeAclsRequest,
        api_version: int,
        callback: Callable[[DescribeAclsResponse], Awaitable[None]],
    ):
        pass

    def describe_acls_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: DescribeAclsRequest,
        api_version: int,
    ) -> DescribeAclsResponse:
        pass

    async def handle_describe_client_quotas_request(
        self,
        header: DescribeClientQuotasRequestHeader,
        req: DescribeClientQuotasRequest,
        api_version: int,
        callback: Callable[[DescribeClientQuotasResponse], Awaitable[None]],
    ):
        pass

    def describe_client_quotas_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: DescribeClientQuotasRequest,
        api_version: int,
    ) -> DescribeClientQuotasResponse:
        pass

    async def handle_describe_cluster_request(
        self,
        header: DescribeClusterRequestHeader,
        req: DescribeClusterRequest,
        api_version: int,
        callback: Callable[[DescribeClusterResponse], Awaitable[None]],
    ):
        pass

    def describe_cluster_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: DescribeClusterRequest,
        api_version: int,
    ) -> DescribeClusterResponse:
        pass

    async def handle_describe_configs_request(
        self,
        header: DescribeConfigsRequestHeader,
        req: DescribeConfigsRequest,
        api_version: int,
        callback: Callable[[DescribeConfigsResponse], Awaitable[None]],
    ):
        pass

    def describe_configs_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: DescribeConfigsRequest,
        api_version: int,
    ) -> DescribeConfigsResponse:
        pass

    async def handle_describe_delegation_token_request(
        self,
        header: DescribeDelegationTokenRequestHeader,
        req: DescribeDelegationTokenRequest,
        api_version: int,
        callback: Callable[[DescribeDelegationTokenResponse], Awaitable[None]],
    ):
        pass

    def describe_delegation_token_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: DescribeDelegationTokenRequest,
        api_version: int,
    ) -> DescribeDelegationTokenResponse:
        pass

    async def handle_describe_groups_request(
        self,
        header: DescribeGroupsRequestHeader,
        req: DescribeGroupsRequest,
        api_version: int,
        callback: Callable[[DescribeGroupsResponse], Awaitable[None]],
    ):
        pass

    def describe_groups_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: DescribeGroupsRequest,
        api_version: int,
    ) -> DescribeGroupsResponse:
        pass

    async def handle_describe_log_dirs_request(
        self,
        header: DescribeLogDirsRequestHeader,
        req: DescribeLogDirsRequest,
        api_version: int,
        callback: Callable[[DescribeLogDirsResponse], Awaitable[None]],
    ):
        pass

    def describe_log_dirs_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: DescribeLogDirsRequest,
        api_version: int,
    ) -> DescribeLogDirsResponse:
        pass

    async def handle_describe_producers_request(
        self,
        header: DescribeProducersRequestHeader,
        req: DescribeProducersRequest,
        api_version: int,
        callback: Callable[[DescribeProducersResponse], Awaitable[None]],
    ):
        pass

    def describe_producers_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: DescribeProducersRequest,
        api_version: int,
    ) -> DescribeProducersResponse:
        pass

    async def handle_describe_quorum_request(
        self,
        header: DescribeQuorumRequestHeader,
        req: DescribeQuorumRequest,
        api_version: int,
        callback: Callable[[DescribeQuorumResponse], Awaitable[None]],
    ):
        pass

    def describe_quorum_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: DescribeQuorumRequest,
        api_version: int,
    ) -> DescribeQuorumResponse:
        pass

    async def handle_describe_topic_partitions_request(
        self,
        header: DescribeTopicPartitionsRequestHeader,
        req: DescribeTopicPartitionsRequest,
        api_version: int,
        callback: Callable[[DescribeTopicPartitionsResponse], Awaitable[None]],
    ):
        pass

    def describe_topic_partitions_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: DescribeTopicPartitionsRequest,
        api_version: int,
    ) -> DescribeTopicPartitionsResponse:
        pass

    async def handle_describe_transactions_request(
        self,
        header: DescribeTransactionsRequestHeader,
        req: DescribeTransactionsRequest,
        api_version: int,
        callback: Callable[[DescribeTransactionsResponse], Awaitable[None]],
    ):
        pass

    def describe_transactions_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: DescribeTransactionsRequest,
        api_version: int,
    ) -> DescribeTransactionsResponse:
        pass

    async def handle_describe_user_scram_credentials_request(
        self,
        header: DescribeUserScramCredentialsRequestHeader,
        req: DescribeUserScramCredentialsRequest,
        api_version: int,
        callback: Callable[[DescribeUserScramCredentialsResponse], Awaitable[None]],
    ):
        pass

    def describe_user_scram_credentials_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: DescribeUserScramCredentialsRequest,
        api_version: int,
    ) -> DescribeUserScramCredentialsResponse:
        pass

    async def handle_elect_leaders_request(
        self,
        header: ElectLeadersRequestHeader,
        req: ElectLeadersRequest,
        api_version: int,
        callback: Callable[[ElectLeadersResponse], Awaitable[None]],
    ):
        pass

    def elect_leaders_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: ElectLeadersRequest,
        api_version: int,
    ) -> ElectLeadersResponse:
        pass

    async def handle_end_quorum_epoch_request(
        self,
        header: EndQuorumEpochRequestHeader,
        req: EndQuorumEpochRequest,
        api_version: int,
        callback: Callable[[EndQuorumEpochResponse], Awaitable[None]],
    ):
        pass

    def end_quorum_epoch_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: EndQuorumEpochRequest,
        api_version: int,
    ) -> EndQuorumEpochResponse:
        pass

    async def handle_end_txn_request(
        self,
        header: EndTxnRequestHeader,
        req: EndTxnRequest,
        api_version: int,
        callback: Callable[[EndTxnResponse], Awaitable[None]],
    ):
        pass

    def end_txn_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: EndTxnRequest,
        api_version: int,
    ) -> EndTxnResponse:
        pass

    async def handle_envelope_request(
        self,
        header: EnvelopeRequestHeader,
        req: EnvelopeRequest,
        api_version: int,
        callback: Callable[[EnvelopeResponse], Awaitable[None]],
    ):
        pass

    def envelope_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: EnvelopeRequest,
        api_version: int,
    ) -> EnvelopeResponse:
        pass

    async def handle_add_offsets_to_txn_request(
            self,
            header: AddOffsetsToTxnRequestHeader,
            req: AddOffsetsToTxnRequest,
            api_version: int,
            callback: Callable[[AddOffsetsToTxnResponse], Awaitable[None]],
    ):
        pass

    def add_offsets_to_txn_request_error_response(
            self,
            error_code: ErrorCode,
            error_message: str,
            req: AddOffsetsToTxnRequest,
            api_version: int,
    ) -> AddOffsetsToTxnResponse:
        pass

    async def handle_add_partitions_to_txn_request(
            self,
            header: AddPartitionsToTxnRequestHeader,
            req: AddPartitionsToTxnRequest,
            api_version: int,
            callback: Callable[[AddPartitionsToTxnResponse], Awaitable[None]],
    ):
        pass

    def add_partitions_to_txn_request_error_response(
            self,
            error_code: ErrorCode,
            error_message: str,
            req: AddPartitionsToTxnRequest,
            api_version: int,
    ) -> AddPartitionsToTxnResponse:
        pass

    async def handle_alter_client_quotas_request(
            self,
            header: AlterClientQuotasRequestHeader,
            req: AlterClientQuotasRequest,
            api_version: int,
            callback: Callable[[AlterClientQuotasResponse], Awaitable[None]],
    ):
        pass

    def alter_client_quotas_request_error_response(
            self,
            error_code: ErrorCode,
            error_message: str,
            req: AlterClientQuotasRequest,
            api_version: int,
    ) -> AlterClientQuotasResponse:
        pass

    async def handle_add_raft_voter_request(
            self,
            header: AddRaftVoterRequestHeader,
            req: AddRaftVoterRequest,
            api_version: int,
            callback: Callable[[AddRaftVoterResponse], Awaitable[None]],
    ):
        pass

    def add_raft_voter_request_error_response(
            self,
            error_code: ErrorCode,
            error_message: str,
            req: AddRaftVoterRequest,
            api_version: int,
    ) -> AddRaftVoterResponse:
        pass

    async def handle_allocate_producer_ids_request(
            self,
            header: AllocateProducerIdsRequestHeader,
            req: AllocateProducerIdsRequest,
            api_version: int,
            callback: Callable[[AllocateProducerIdsResponse], Awaitable[None]],
    ):
        pass

    def allocate_producer_ids_request_error_response(
            self,
            error_code: ErrorCode,
            error_message: str,
            req: AllocateProducerIdsRequest,
            api_version: int,
    ) -> AllocateProducerIdsResponse:
        pass

    async def handle_alter_configs_request(
            self,
            header: AlterConfigsRequestHeader,
            req: AlterConfigsRequest,
            api_version: int,
            callback: Callable[[AlterConfigsResponse], Awaitable[None]],
    ):
        pass

    def alter_configs_request_error_response(
            self,
            error_code: ErrorCode,
            error_message: str,
            req: AlterConfigsRequest,
            api_version: int,
    ) -> AlterConfigsResponse:
        pass

    async def handle_alter_partition_request(
            self,
            header: AlterPartitionRequestHeader,
            req: AlterPartitionRequest,
            api_version: int,
            callback: Callable[[AlterPartitionResponse], Awaitable[None]],
    ):
        pass

    def alter_partition_request_error_response(
            self,
            error_code: ErrorCode,
            error_message: str,
            req: AlterPartitionRequest,
            api_version: int,
    ) -> AlterPartitionResponse:
        pass

    async def handle_alter_partition_reassignments_request(
            self,
            header: AlterPartitionReassignmentsRequestHeader,
            req: AlterPartitionReassignmentsRequest,
            api_version: int,
            callback: Callable[[AlterPartitionReassignmentsResponse], Awaitable[None]],
    ):
        pass

    def alter_partition_reassignments_request_error_response(
            self,
            error_code: ErrorCode,
            error_message: str,
            req: AlterPartitionReassignmentsRequest,
            api_version: int,
    ) -> AlterPartitionReassignmentsResponse:
        pass

    async def handle_alter_replica_log_dirs_request(
            self,
            header: AlterReplicaLogDirsRequestHeader,
            req: AlterReplicaLogDirsRequest,
            api_version: int,
            callback: Callable[[AlterReplicaLogDirsResponse], Awaitable[None]],
    ):
        pass

    def alter_replica_log_dirs_request_error_response(
            self,
            error_code: ErrorCode,
            error_message: str,
            req: AlterReplicaLogDirsRequest,
            api_version: int,
    ) -> AlterReplicaLogDirsResponse:
        pass

    async def handle_alter_user_scram_credentials_request(
            self,
            header: AlterUserScramCredentialsRequestHeader,
            req: AlterUserScramCredentialsRequest,
            api_version: int,
            callback: Callable[[AlterUserScramCredentialsResponse], Awaitable[None]],
    ):
        pass

    def alter_user_scram_credentials_request_error_response(
            self,
            error_code: ErrorCode,
            error_message: str,
            req: AlterUserScramCredentialsRequest,
            api_version: int,
    ) -> AlterUserScramCredentialsResponse:
        pass

    async def handle_assign_replicas_to_dirs_request(
            self,
            header: AssignReplicasToDirsRequestHeader,
            req: AssignReplicasToDirsRequest,
            api_version: int,
            callback: Callable[[AssignReplicasToDirsResponse], Awaitable[None]],
    ):
        pass

    def assign_replicas_to_dirs_request_error_response(
            self,
            error_code: ErrorCode,
            error_message: str,
            req: AssignReplicasToDirsRequest,
            api_version: int,
    ) -> AssignReplicasToDirsResponse:
        pass

    async def handle_begin_quorum_epoch_request(
            self,
            header: BeginQuorumEpochRequestHeader,
            req: BeginQuorumEpochRequest,
            api_version: int,
            callback: Callable[[BeginQuorumEpochResponse], Awaitable[None]],
    ):
        pass

    def begin_quorum_epoch_request_error_response(
            self,
            error_code: ErrorCode,
            error_message: str,
            req: BeginQuorumEpochRequest,
            api_version: int,
    ) -> BeginQuorumEpochResponse:
        pass

    async def handle_broker_heartbeat_request(
            self,
            header: BrokerHeartbeatRequestHeader,
            req: BrokerHeartbeatRequest,
            api_version: int,
            callback: Callable[[BrokerHeartbeatResponse], Awaitable[None]],
    ):
        pass

    def broker_heartbeat_request_error_response(
            self,
            error_code: ErrorCode,
            error_message: str,
            req: BrokerHeartbeatRequest,
            api_version: int,
    ) -> BrokerHeartbeatResponse:
        pass

    async def handle_broker_registration_request(
            self,
            header: BrokerRegistrationRequestHeader,
            req: BrokerRegistrationRequest,
            api_version: int,
            callback: Callable[[BrokerRegistrationResponse], Awaitable[None]],
    ):
        pass

    def broker_registration_request_error_response(
            self,
            error_code: ErrorCode,
            error_message: str,
            req: BrokerRegistrationRequest,
            api_version: int,
    ) -> BrokerRegistrationResponse:
        pass

    async def handle_consumer_group_describe_request(
            self,
            header: ConsumerGroupDescribeRequestHeader,
            req: ConsumerGroupDescribeRequest,
            api_version: int,
            callback: Callable[[ConsumerGroupDescribeResponse], Awaitable[None]],
    ):
        pass

    def consumer_group_describe_request_error_response(
            self,
            error_code: ErrorCode,
            error_message: str,
            req: ConsumerGroupDescribeRequest,
            api_version: int,
    ) -> ConsumerGroupDescribeResponse:
        pass

    async def handle_consumer_group_heartbeat_request(
            self,
            header: ConsumerGroupHeartbeatRequestHeader,
            req: ConsumerGroupHeartbeatRequest,
            api_version: int,
            callback: Callable[[ConsumerGroupHeartbeatResponse], Awaitable[None]],
    ):
        pass

    def consumer_group_heartbeat_request_error_response(
            self,
            error_code: ErrorCode,
            error_message: str,
            req: ConsumerGroupHeartbeatRequest,
            api_version: int,
    ) -> ConsumerGroupHeartbeatResponse:
        pass

    async def handle_controlled_shutdown_request(
            self,
            header: ControlledShutdownRequestHeader,
            req: ControlledShutdownRequest,
            api_version: int,
            callback: Callable[[ControlledShutdownResponse], Awaitable[None]],
    ):
        pass

    def controlled_shutdown_request_error_response(
            self,
            error_code: ErrorCode,
            error_message: str,
            req: ControlledShutdownRequest,
            api_version: int,
    ) -> ControlledShutdownResponse:
        pass

    async def handle_controller_registration_request(
            self,
            header: ControllerRegistrationRequestHeader,
            req: ControllerRegistrationRequest,
            api_version: int,
            callback: Callable[[ControllerRegistrationResponse], Awaitable[None]],
    ):
        pass

    def controller_registration_request_error_response(
            self,
            error_code: ErrorCode,
            error_message: str,
            req: ControllerRegistrationRequest,
            api_version: int,
    ) -> ControllerRegistrationResponse:
        pass

    async def handle_create_acls_request(
        self,
        header: CreateAclsRequestHeader,
        req: CreateAclsRequest,
        api_version: int,
        callback: Callable[[CreateAclsResponse], Awaitable[None]],
    ):
        pass

    def create_acls_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: CreateAclsRequest,
        api_version: int,
    ) -> CreateAclsResponse:
        pass

    async def handle_create_delegation_token_request(
        self,
        header: CreateDelegationTokenRequestHeader,
        req: CreateDelegationTokenRequest,
        api_version: int,
        callback: Callable[[CreateDelegationTokenResponse], Awaitable[None]],
    ):
        pass

    def create_delegation_token_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: CreateDelegationTokenRequest,
        api_version: int,
    ) -> CreateDelegationTokenResponse:
        pass

    async def handle_create_partitions_request(
        self,
        header: CreatePartitionsRequestHeader,
        req: CreatePartitionsRequest,
        api_version: int,
        callback: Callable[[CreatePartitionsResponse], Awaitable[None]],
    ):
        pass

    def create_partitions_request_error_response(
        self,
        error_code: ErrorCode,
        error_message: str,
        req: CreatePartitionsRequest,
        api_version: int,
    ) -> CreatePartitionsResponse:
        pass



from kio.schema.create_acls.v0.request import CreateAclsRequest as CreateAclsRequestV0
from kio.schema.create_acls.v0.request import (
    RequestHeader as CreateAclsRequestHeaderV0,
)
from kio.schema.create_acls.v0.response import CreateAclsResponse as CreateAclsResponseV0
from kio.schema.create_acls.v0.response import (
    ResponseHeader as CreateAclsResponseHeaderV0,
)
from kio.schema.create_acls.v1.request import CreateAclsRequest as CreateAclsRequestV1
from kio.schema.create_acls.v1.request import (
    RequestHeader as CreateAclsRequestHeaderV1,
)
from kio.schema.create_acls.v1.response import CreateAclsResponse as CreateAclsResponseV1
from kio.schema.create_acls.v1.response import (
    ResponseHeader as CreateAclsResponseHeaderV1,
)
from kio.schema.create_acls.v2.request import CreateAclsRequest as CreateAclsRequestV2
from kio.schema.create_acls.v2.request import (
    RequestHeader as CreateAclsRequestHeaderV2,
)
from kio.schema.create_acls.v2.response import CreateAclsResponse as CreateAclsResponseV2
from kio.schema.create_acls.v2.response import (
    ResponseHeader as CreateAclsResponseHeaderV2,
)
from kio.schema.create_acls.v3.request import CreateAclsRequest as CreateAclsRequestV3
from kio.schema.create_acls.v3.request import (
    RequestHeader as CreateAclsRequestHeaderV3,
)
from kio.schema.create_acls.v3.response import CreateAclsResponse as CreateAclsResponseV3
from kio.schema.create_acls.v3.response import (
    ResponseHeader as CreateAclsResponseHeaderV3,
)

CreateAclsRequestHeader = (
    CreateAclsRequestHeaderV0
    | CreateAclsRequestHeaderV1
    | CreateAclsRequestHeaderV2
    | CreateAclsRequestHeaderV3
)

CreateAclsResponseHeader = (
    CreateAclsResponseHeaderV0
    | CreateAclsResponseHeaderV1
    | CreateAclsResponseHeaderV2
    | CreateAclsResponseHeaderV3
)

CreateAclsRequest = (
    CreateAclsRequestV0
    | CreateAclsRequestV1
    | CreateAclsRequestV2
    | CreateAclsRequestV3
)

CreateAclsResponse = (
    CreateAclsResponseV0
    | CreateAclsResponseV1
    | CreateAclsResponseV2
    | CreateAclsResponseV3
)



from kio.schema.alter_user_scram_credentials.v0.request import (
    AlterUserScramCredentialsRequest as AlterUserScramCredentialsRequestV0,
)
from kio.schema.alter_user_scram_credentials.v0.request import (
    RequestHeader as AlterUserScramCredentialsRequestHeaderV0,
)
from kio.schema.alter_user_scram_credentials.v0.response import (
    AlterUserScramCredentialsResponse as AlterUserScramCredentialsResponseV0,
)
from kio.schema.alter_user_scram_credentials.v0.response import (
    ResponseHeader as AlterUserScramCredentialsResponseHeaderV0,
)

AlterUserScramCredentialsRequestHeader = AlterUserScramCredentialsRequestHeaderV0
AlterUserScramCredentialsResponseHeader = AlterUserScramCredentialsResponseHeaderV0
AlterUserScramCredentialsRequest = AlterUserScramCredentialsRequestV0
AlterUserScramCredentialsResponse = AlterUserScramCredentialsResponseV0



from kio.schema.describe_cluster.v0.request import (
    DescribeClusterRequest as DescribeClusterRequestV0,
)
from kio.schema.describe_cluster.v0.request import (
    RequestHeader as DescribeClusterRequestHeaderV0,
)
from kio.schema.describe_cluster.v0.response import (
    DescribeClusterResponse as DescribeClusterResponseV0,
)
from kio.schema.describe_cluster.v0.response import (
    ResponseHeader as DescribeClusterResponseHeaderV0,
)
from kio.schema.describe_cluster.v1.request import (
    DescribeClusterRequest as DescribeClusterRequestV1,
)
from kio.schema.describe_cluster.v1.request import (
    RequestHeader as DescribeClusterRequestHeaderV1,
)
from kio.schema.describe_cluster.v1.response import (
    DescribeClusterResponse as DescribeClusterResponseV1,
)
from kio.schema.describe_cluster.v1.response import (
    ResponseHeader as DescribeClusterResponseHeaderV1,
)

DescribeClusterRequestHeader = (
    DescribeClusterRequestHeaderV0
    | DescribeClusterRequestHeaderV1
)

DescribeClusterResponseHeader = (
    DescribeClusterResponseHeaderV0
    | DescribeClusterResponseHeaderV1
)

DescribeClusterRequest = (
    DescribeClusterRequestV0
    | DescribeClusterRequestV1
)

DescribeClusterResponse = (
    DescribeClusterResponseV0
    | DescribeClusterResponseV1
)



from kio.schema.elect_leaders.v0.request import (
    ElectLeadersRequest as ElectLeadersRequestV0,
)
from kio.schema.elect_leaders.v0.request import (
    RequestHeader as ElectLeadersRequestHeaderV0,
)
from kio.schema.elect_leaders.v0.response import (
    ElectLeadersResponse as ElectLeadersResponseV0,
)
from kio.schema.elect_leaders.v0.response import (
    ResponseHeader as ElectLeadersResponseHeaderV0,
)
from kio.schema.elect_leaders.v1.request import (
    ElectLeadersRequest as ElectLeadersRequestV1,
)
from kio.schema.elect_leaders.v1.request import (
    RequestHeader as ElectLeadersRequestHeaderV1,
)
from kio.schema.elect_leaders.v1.response import (
    ElectLeadersResponse as ElectLeadersResponseV1,
)
from kio.schema.elect_leaders.v1.response import (
    ResponseHeader as ElectLeadersResponseHeaderV1,
)
from kio.schema.elect_leaders.v2.request import (
    ElectLeadersRequest as ElectLeadersRequestV2,
)
from kio.schema.elect_leaders.v2.request import (
    RequestHeader as ElectLeadersRequestHeaderV2,
)
from kio.schema.elect_leaders.v2.response import (
    ElectLeadersResponse as ElectLeadersResponseV2,
)
from kio.schema.elect_leaders.v2.response import (
    ResponseHeader as ElectLeadersResponseHeaderV2,
)

ElectLeadersRequestHeader = (
    ElectLeadersRequestHeaderV0
    | ElectLeadersRequestHeaderV1
    | ElectLeadersRequestHeaderV2
)

ElectLeadersResponseHeader = (
    ElectLeadersResponseHeaderV0
    | ElectLeadersResponseHeaderV1
    | ElectLeadersResponseHeaderV2
)

ElectLeadersRequest = (
    ElectLeadersRequestV0
    | ElectLeadersRequestV1
    | ElectLeadersRequestV2
)

ElectLeadersResponse = (
    ElectLeadersResponseV0
    | ElectLeadersResponseV1
    | ElectLeadersResponseV2
)



"""remove unnecessary compaction columns from wal file

Revision ID: 298875b23096
Revises: d77fb56536a8
Create Date: 2025-07-14 10:46:10.006756

"""

from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision: str = "298875b23096"
down_revision: Union[str, None] = "d77fb56536a8"
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    """Upgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("wal_files", "compacted_by")
    op.drop_column("wal_files", "compacted")
    # ### end Alembic commands ###


def downgrade() -> None:
    """Downgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "wal_files",
        sa.Column("compacted", sa.BOOLEAN(), autoincrement=False, nullable=False),
    )
    op.add_column(
        "wal_files",
        sa.Column("compacted_by", sa.TEXT(), autoincrement=False, nullable=True),
    )
    # ### end Alembic commands ###



"""initial schema

Revision ID: bf37d72775bc
Revises:
Create Date: 2025-06-25 14:55:14.394624

"""

from typing import Sequence, Union

import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision: str = "bf37d72775bc"
down_revision: Union[str, None] = None
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    """Upgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "topics",
        sa.Column("name", sa.String(), nullable=False),
        sa.Column("id", sa.Integer(), sa.Identity(always=True), nullable=False),
        sa.Column(
            "created_at",
            sa.TIMESTAMP(timezone=True),
            server_default=sa.text("CURRENT_TIMESTAMP"),
            nullable=False,
        ),
        sa.Column(
            "updated_at",
            sa.TIMESTAMP(timezone=True),
            server_default=sa.text("CURRENT_TIMESTAMP"),
            nullable=False,
        ),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("name"),
    )
    op.create_table(
        "wal_files",
        sa.Column("s3_uri", sa.Text(), nullable=False),
        sa.Column("s3_etag", sa.Text(), nullable=True),
        sa.Column("total_bytes", sa.BigInteger(), nullable=False),
        sa.Column("total_messages", sa.Integer(), nullable=False),
        sa.Column("compacted", sa.Boolean(), nullable=False),
        sa.Column("compacted_at", sa.TIMESTAMP(timezone=True), nullable=True),
        sa.Column("compacted_by", sa.Text(), nullable=True),
        sa.Column("iceberg_commit_id", sa.Text(), nullable=True),
        sa.Column("id", sa.BigInteger(), sa.Identity(always=True), nullable=False),
        sa.Column(
            "created_at",
            sa.TIMESTAMP(timezone=True),
            server_default=sa.text("CURRENT_TIMESTAMP"),
            nullable=False,
        ),
        sa.Column(
            "updated_at",
            sa.TIMESTAMP(timezone=True),
            server_default=sa.text("CURRENT_TIMESTAMP"),
            nullable=False,
        ),
        sa.PrimaryKeyConstraint("id"),
    )
    op.create_table(
        "partitions",
        sa.Column("topic_name", sa.String(), nullable=False),
        sa.Column("partition_number", sa.Integer(), nullable=False),
        sa.Column("last_offset", sa.BigInteger(), nullable=False),
        sa.Column("id", sa.Integer(), sa.Identity(always=True), nullable=False),
        sa.Column(
            "created_at",
            sa.TIMESTAMP(timezone=True),
            server_default=sa.text("CURRENT_TIMESTAMP"),
            nullable=False,
        ),
        sa.Column(
            "updated_at",
            sa.TIMESTAMP(timezone=True),
            server_default=sa.text("CURRENT_TIMESTAMP"),
            nullable=False,
        ),
        sa.ForeignKeyConstraint(["topic_name"], ["topics.name"], ondelete="CASCADE"),
        sa.PrimaryKeyConstraint("id"),
        sa.UniqueConstraint("topic_name", "partition_number"),
    )
    op.create_index(
        "ix_partition_topic_partition",
        "partitions",
        ["topic_name", "partition_number"],
        unique=False,
    )
    op.create_table(
        "wal_file_offsets",
        sa.Column("wal_file_id", sa.BigInteger(), nullable=False),
        sa.Column("partition_id", sa.Integer(), nullable=False),
        sa.Column("base_offset", sa.BigInteger(), nullable=False),
        sa.Column("last_offset", sa.BigInteger(), nullable=False),
        sa.Column("id", sa.BigInteger(), sa.Identity(always=True), nullable=False),
        sa.ForeignKeyConstraint(
            ["partition_id"], ["partitions.id"], ondelete="CASCADE"
        ),
        sa.ForeignKeyConstraint(["wal_file_id"], ["wal_files.id"], ondelete="CASCADE"),
        sa.PrimaryKeyConstraint("id"),
    )
    # ### end Alembic commands ###


def downgrade() -> None:
    """Downgrade schema."""
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table("wal_file_offsets")
    op.drop_index("ix_partition_topic_partition", table_name="partitions")
    op.drop_table("partitions")
    op.drop_table("wal_files")
    op.drop_table("topics")
    # ### end Alembic commands ###



from typing import Any, Callable

from kio.schema.metadata.v0.request import (
    MetadataRequest as MetadataRequestV0,
)
from kio.schema.metadata.v0.request import (
    RequestHeader as MetadataRequestHeaderV0,
)
from kio.schema.metadata.v0.response import (
    MetadataResponse as MetadataResponseV0,
)
from kio.schema.metadata.v1.request import (
    MetadataRequest as MetadataRequestV1,
)
from kio.schema.metadata.v1.request import (
    RequestHeader as MetadataRequestHeaderV1,
)
from kio.schema.metadata.v1.response import (
    MetadataResponse as MetadataResponseV1,
)
from kio.schema.metadata.v2.request import (
    MetadataRequest as MetadataRequestV2,
)
from kio.schema.metadata.v2.request import (
    RequestHeader as MetadataRequestHeaderV2,
)
from kio.schema.metadata.v2.response import (
    MetadataResponse as MetadataResponseV2,
)
from kio.schema.metadata.v3.request import (
    MetadataRequest as MetadataRequestV3,
)
from kio.schema.metadata.v3.request import (
    RequestHeader as MetadataRequestHeaderV3,
)
from kio.schema.metadata.v3.response import (
    MetadataResponse as MetadataResponseV3,
)
from kio.schema.metadata.v4.request import (
    MetadataRequest as MetadataRequestV4,
)
from kio.schema.metadata.v4.request import (
    RequestHeader as MetadataRequestHeaderV4,
)
from kio.schema.metadata.v4.response import (
    MetadataResponse as MetadataResponseV4,
)
from kio.schema.metadata.v5.request import (
    MetadataRequest as MetadataRequestV5,
)
from kio.schema.metadata.v5.request import (
    RequestHeader as MetadataRequestHeaderV5,
)
from kio.schema.metadata.v5.response import (
    MetadataResponse as MetadataResponseV5,
)
from kio.schema.metadata.v6.request import (
    MetadataRequest as MetadataRequestV6,
)
from kio.schema.metadata.v6.request import (
    RequestHeader as MetadataRequestHeaderV6,
)
from kio.schema.metadata.v6.response import (
    MetadataResponse as MetadataResponseV6,
)

MetadataRequestHeader = (
    MetadataRequestHeaderV0
    | MetadataRequestHeaderV1
    | MetadataRequestHeaderV2
    | MetadataRequestHeaderV3
    | MetadataRequestHeaderV4
    | MetadataRequestHeaderV5
    | MetadataRequestHeaderV6
)

MetadataRequest = (
    MetadataRequestV0
    | MetadataRequestV1
    | MetadataRequestV2
    | MetadataRequestV3
    | MetadataRequestV4
    | MetadataRequestV5
    | MetadataRequestV6
)

MetadataResponse = (
    MetadataResponseV0
    | MetadataResponseV1
    | MetadataResponseV2
    | MetadataResponseV3
    | MetadataResponseV4
    | MetadataResponseV5
    | MetadataResponseV6
)


async def do_handle_metadata_request(
    header: MetadataRequestHeader,
    req: MetadataRequest,
    api_version: int,
    callback: Callable[[MetadataResponse], Any],
):
    pass



from kio.schema.add_partitions_to_txn.v0.request import (
    AddPartitionsToTxnRequest as AddPartitionsToTxnRequestV0,
)
from kio.schema.add_partitions_to_txn.v0.request import (
    RequestHeader as AddPartitionsToTxnRequestHeaderV0,
)
from kio.schema.add_partitions_to_txn.v0.response import (
    AddPartitionsToTxnResponse as AddPartitionsToTxnResponseV0,
)
from kio.schema.add_partitions_to_txn.v0.response import (
    ResponseHeader as AddPartitionsToTxnResponseHeaderV0,
)
from kio.schema.add_partitions_to_txn.v1.request import (
    AddPartitionsToTxnRequest as AddPartitionsToTxnRequestV1,
)
from kio.schema.add_partitions_to_txn.v1.request import (
    RequestHeader as AddPartitionsToTxnRequestHeaderV1,
)
from kio.schema.add_partitions_to_txn.v1.response import (
    AddPartitionsToTxnResponse as AddPartitionsToTxnResponseV1,
)
from kio.schema.add_partitions_to_txn.v1.response import (
    ResponseHeader as AddPartitionsToTxnResponseHeaderV1,
)
from kio.schema.add_partitions_to_txn.v2.request import (
    AddPartitionsToTxnRequest as AddPartitionsToTxnRequestV2,
)
from kio.schema.add_partitions_to_txn.v2.request import (
    RequestHeader as AddPartitionsToTxnRequestHeaderV2,
)
from kio.schema.add_partitions_to_txn.v2.response import (
    AddPartitionsToTxnResponse as AddPartitionsToTxnResponseV2,
)
from kio.schema.add_partitions_to_txn.v2.response import (
    ResponseHeader as AddPartitionsToTxnResponseHeaderV2,
)
from kio.schema.add_partitions_to_txn.v3.request import (
    AddPartitionsToTxnRequest as AddPartitionsToTxnRequestV3,
)
from kio.schema.add_partitions_to_txn.v3.request import (
    RequestHeader as AddPartitionsToTxnRequestHeaderV3,
)
from kio.schema.add_partitions_to_txn.v3.response import (
    AddPartitionsToTxnResponse as AddPartitionsToTxnResponseV3,
)
from kio.schema.add_partitions_to_txn.v3.response import (
    ResponseHeader as AddPartitionsToTxnResponseHeaderV3,
)
from kio.schema.add_partitions_to_txn.v4.request import (
    AddPartitionsToTxnRequest as AddPartitionsToTxnRequestV4,
)
from kio.schema.add_partitions_to_txn.v4.request import (
    RequestHeader as AddPartitionsToTxnRequestHeaderV4,
)
from kio.schema.add_partitions_to_txn.v4.response import (
    AddPartitionsToTxnResponse as AddPartitionsToTxnResponseV4,
)
from kio.schema.add_partitions_to_txn.v4.response import (
    ResponseHeader as AddPartitionsToTxnResponseHeaderV4,
)
from kio.schema.add_partitions_to_txn.v5.request import (
    AddPartitionsToTxnRequest as AddPartitionsToTxnRequestV5,
)
from kio.schema.add_partitions_to_txn.v5.request import (
    RequestHeader as AddPartitionsToTxnRequestHeaderV5,
)
from kio.schema.add_partitions_to_txn.v5.response import (
    AddPartitionsToTxnResponse as AddPartitionsToTxnResponseV5,
)
from kio.schema.add_partitions_to_txn.v5.response import (
    ResponseHeader as AddPartitionsToTxnResponseHeaderV5,
)

AddPartitionsToTxnRequestHeader = (
    AddPartitionsToTxnRequestHeaderV0
    | AddPartitionsToTxnRequestHeaderV1
    | AddPartitionsToTxnRequestHeaderV2
    | AddPartitionsToTxnRequestHeaderV3
    | AddPartitionsToTxnRequestHeaderV4
    | AddPartitionsToTxnRequestHeaderV5
)

AddPartitionsToTxnResponseHeader = (
    AddPartitionsToTxnResponseHeaderV0
    | AddPartitionsToTxnResponseHeaderV1
    | AddPartitionsToTxnResponseHeaderV2
    | AddPartitionsToTxnResponseHeaderV3
    | AddPartitionsToTxnResponseHeaderV4
    | AddPartitionsToTxnResponseHeaderV5
)

AddPartitionsToTxnRequest = (
    AddPartitionsToTxnRequestV0
    | AddPartitionsToTxnRequestV1
    | AddPartitionsToTxnRequestV2
    | AddPartitionsToTxnRequestV3
    | AddPartitionsToTxnRequestV4
    | AddPartitionsToTxnRequestV5
)

AddPartitionsToTxnResponse = (
    AddPartitionsToTxnResponseV0
    | AddPartitionsToTxnResponseV1
    | AddPartitionsToTxnResponseV2
    | AddPartitionsToTxnResponseV3
    | AddPartitionsToTxnResponseV4
    | AddPartitionsToTxnResponseV5
)



from kio.schema.add_offsets_to_txn.v0.request import (
    AddOffsetsToTxnRequest as AddOffsetsToTxnRequestV0,
)
from kio.schema.add_offsets_to_txn.v0.request import (
    RequestHeader as AddOffsetsToTxnRequestHeaderV0,
)
from kio.schema.add_offsets_to_txn.v0.response import (
    AddOffsetsToTxnResponse as AddOffsetsToTxnResponseV0,
)
from kio.schema.add_offsets_to_txn.v0.response import (
    ResponseHeader as AddOffsetsToTxnResponseHeaderV0,
)
from kio.schema.add_offsets_to_txn.v1.request import (
    AddOffsetsToTxnRequest as AddOffsetsToTxnRequestV1,
)
from kio.schema.add_offsets_to_txn.v1.request import (
    RequestHeader as AddOffsetsToTxnRequestHeaderV1,
)
from kio.schema.add_offsets_to_txn.v1.response import (
    AddOffsetsToTxnResponse as AddOffsetsToTxnResponseV1,
)
from kio.schema.add_offsets_to_txn.v1.response import (
    ResponseHeader as AddOffsetsToTxnResponseHeaderV1,
)
from kio.schema.add_offsets_to_txn.v2.request import (
    AddOffsetsToTxnRequest as AddOffsetsToTxnRequestV2,
)
from kio.schema.add_offsets_to_txn.v2.request import (
    RequestHeader as AddOffsetsToTxnRequestHeaderV2,
)
from kio.schema.add_offsets_to_txn.v2.response import (
    AddOffsetsToTxnResponse as AddOffsetsToTxnResponseV2,
)
from kio.schema.add_offsets_to_txn.v2.response import (
    ResponseHeader as AddOffsetsToTxnResponseHeaderV2,
)
from kio.schema.add_offsets_to_txn.v3.request import (
    AddOffsetsToTxnRequest as AddOffsetsToTxnRequestV3,
)
from kio.schema.add_offsets_to_txn.v3.request import (
    RequestHeader as AddOffsetsToTxnRequestHeaderV3,
)
from kio.schema.add_offsets_to_txn.v3.response import (
    AddOffsetsToTxnResponse as AddOffsetsToTxnResponseV3,
)
from kio.schema.add_offsets_to_txn.v3.response import (
    ResponseHeader as AddOffsetsToTxnResponseHeaderV3,
)
from kio.schema.add_offsets_to_txn.v4.request import (
    AddOffsetsToTxnRequest as AddOffsetsToTxnRequestV4,
)
from kio.schema.add_offsets_to_txn.v4.request import (
    RequestHeader as AddOffsetsToTxnRequestHeaderV4,
)
from kio.schema.add_offsets_to_txn.v4.response import (
    AddOffsetsToTxnResponse as AddOffsetsToTxnResponseV4,
)
from kio.schema.add_offsets_to_txn.v4.response import (
    ResponseHeader as AddOffsetsToTxnResponseHeaderV4,
)

AddOffsetsToTxnRequestHeader = (
    AddOffsetsToTxnRequestHeaderV0
    | AddOffsetsToTxnRequestHeaderV1
    | AddOffsetsToTxnRequestHeaderV2
    | AddOffsetsToTxnRequestHeaderV3
    | AddOffsetsToTxnRequestHeaderV4
)

AddOffsetsToTxnResponseHeader = (
    AddOffsetsToTxnResponseHeaderV0
    | AddOffsetsToTxnResponseHeaderV1
    | AddOffsetsToTxnResponseHeaderV2
    | AddOffsetsToTxnResponseHeaderV3
    | AddOffsetsToTxnResponseHeaderV4
)

AddOffsetsToTxnRequest = (
    AddOffsetsToTxnRequestV0
    | AddOffsetsToTxnRequestV1
    | AddOffsetsToTxnRequestV2
    | AddOffsetsToTxnRequestV3
    | AddOffsetsToTxnRequestV4
)

AddOffsetsToTxnResponse = (
    AddOffsetsToTxnResponseV0
    | AddOffsetsToTxnResponseV1
    | AddOffsetsToTxnResponseV2
    | AddOffsetsToTxnResponseV3
    | AddOffsetsToTxnResponseV4
)



from kio.schema.leave_group.v0.request import (
    LeaveGroupRequest as LeaveGroupRequestV0,
)
from kio.schema.leave_group.v0.request import (
    RequestHeader as LeaveGroupRequestHeaderV0,
)
from kio.schema.leave_group.v0.response import (
    LeaveGroupResponse as LeaveGroupResponseV0,
)
from kio.schema.leave_group.v0.response import (
    ResponseHeader as LeaveGroupResponseHeaderV0,
)
from kio.schema.leave_group.v1.request import (
    LeaveGroupRequest as LeaveGroupRequestV1,
)
from kio.schema.leave_group.v1.request import (
    RequestHeader as LeaveGroupRequestHeaderV1,
)
from kio.schema.leave_group.v1.response import (
    LeaveGroupResponse as LeaveGroupResponseV1,
)
from kio.schema.leave_group.v1.response import (
    ResponseHeader as LeaveGroupResponseHeaderV1,
)
from kio.schema.leave_group.v2.request import (
    LeaveGroupRequest as LeaveGroupRequestV2,
)
from kio.schema.leave_group.v2.request import (
    RequestHeader as LeaveGroupRequestHeaderV2,
)
from kio.schema.leave_group.v2.response import (
    LeaveGroupResponse as LeaveGroupResponseV2,
)
from kio.schema.leave_group.v2.response import (
    ResponseHeader as LeaveGroupResponseHeaderV2,
)
from kio.schema.leave_group.v3.request import (
    LeaveGroupRequest as LeaveGroupRequestV3,
)
from kio.schema.leave_group.v3.request import (
    RequestHeader as LeaveGroupRequestHeaderV3,
)
from kio.schema.leave_group.v3.response import (
    LeaveGroupResponse as LeaveGroupResponseV3,
)
from kio.schema.leave_group.v3.response import (
    ResponseHeader as LeaveGroupResponseHeaderV3,
)
from kio.schema.leave_group.v4.request import (
    LeaveGroupRequest as LeaveGroupRequestV4,
)
from kio.schema.leave_group.v4.request import (
    RequestHeader as LeaveGroupRequestHeaderV4,
)
from kio.schema.leave_group.v4.response import (
    LeaveGroupResponse as LeaveGroupResponseV4,
)
from kio.schema.leave_group.v4.response import (
    ResponseHeader as LeaveGroupResponseHeaderV4,
)
from kio.schema.leave_group.v5.request import (
    LeaveGroupRequest as LeaveGroupRequestV5,
)
from kio.schema.leave_group.v5.request import (
    RequestHeader as LeaveGroupRequestHeaderV5,
)
from kio.schema.leave_group.v5.response import (
    LeaveGroupResponse as LeaveGroupResponseV5,
)
from kio.schema.leave_group.v5.response import (
    ResponseHeader as LeaveGroupResponseHeaderV5,
)


LeaveGroupRequestHeader = (
    LeaveGroupRequestHeaderV0
    | LeaveGroupRequestHeaderV1
    | LeaveGroupRequestHeaderV2
    | LeaveGroupRequestHeaderV3
    | LeaveGroupRequestHeaderV4
    | LeaveGroupRequestHeaderV5
)

LeaveGroupResponseHeader = (
    LeaveGroupResponseHeaderV0
    | LeaveGroupResponseHeaderV1
    | LeaveGroupResponseHeaderV2
    | LeaveGroupResponseHeaderV3
    | LeaveGroupResponseHeaderV4
    | LeaveGroupResponseHeaderV5
)

LeaveGroupRequest = (
    LeaveGroupRequestV0
    | LeaveGroupRequestV1
    | LeaveGroupRequestV2
    | LeaveGroupRequestV3
    | LeaveGroupRequestV4
    | LeaveGroupRequestV5
)

LeaveGroupResponse = (
    LeaveGroupResponseV0
    | LeaveGroupResponseV1
    | LeaveGroupResponseV2
    | LeaveGroupResponseV3
    | LeaveGroupResponseV4
    | LeaveGroupResponseV5
)



from kio.schema.end_txn.v0.request import (
    EndTxnRequest as EndTxnRequestV0,
)
from kio.schema.end_txn.v0.request import (
    RequestHeader as EndTxnRequestHeaderV0,
)
from kio.schema.end_txn.v0.response import (
    EndTxnResponse as EndTxnResponseV0,
)
from kio.schema.end_txn.v0.response import (
    ResponseHeader as EndTxnResponseHeaderV0,
)
from kio.schema.end_txn.v1.request import (
    EndTxnRequest as EndTxnRequestV1,
)
from kio.schema.end_txn.v1.request import (
    RequestHeader as EndTxnRequestHeaderV1,
)
from kio.schema.end_txn.v1.response import (
    EndTxnResponse as EndTxnResponseV1,
)
from kio.schema.end_txn.v1.response import (
    ResponseHeader as EndTxnResponseHeaderV1,
)
from kio.schema.end_txn.v2.request import (
    EndTxnRequest as EndTxnRequestV2,
)
from kio.schema.end_txn.v2.request import (
    RequestHeader as EndTxnRequestHeaderV2,
)
from kio.schema.end_txn.v2.response import (
    EndTxnResponse as EndTxnResponseV2,
)
from kio.schema.end_txn.v2.response import (
    ResponseHeader as EndTxnResponseHeaderV2,
)
from kio.schema.end_txn.v3.request import (
    EndTxnRequest as EndTxnRequestV3,
)
from kio.schema.end_txn.v3.request import (
    RequestHeader as EndTxnRequestHeaderV3,
)
from kio.schema.end_txn.v3.response import (
    EndTxnResponse as EndTxnResponseV3,
)
from kio.schema.end_txn.v3.response import (
    ResponseHeader as EndTxnResponseHeaderV3,
)
from kio.schema.end_txn.v4.request import (
    EndTxnRequest as EndTxnRequestV4,
)
from kio.schema.end_txn.v4.request import (
    RequestHeader as EndTxnRequestHeaderV4,
)
from kio.schema.end_txn.v4.response import (
    EndTxnResponse as EndTxnResponseV4,
)
from kio.schema.end_txn.v4.response import (
    ResponseHeader as EndTxnResponseHeaderV4,
)

EndTxnRequestHeader = (
    EndTxnRequestHeaderV0
    | EndTxnRequestHeaderV1
    | EndTxnRequestHeaderV2
    | EndTxnRequestHeaderV3
    | EndTxnRequestHeaderV4
)

EndTxnResponseHeader = (
    EndTxnResponseHeaderV0
    | EndTxnResponseHeaderV1
    | EndTxnResponseHeaderV2
    | EndTxnResponseHeaderV3
    | EndTxnResponseHeaderV4
)

EndTxnRequest = (
    EndTxnRequestV0
    | EndTxnRequestV1
    | EndTxnRequestV2
    | EndTxnRequestV3
    | EndTxnRequestV4
)

EndTxnResponse = (
    EndTxnResponseV0
    | EndTxnResponseV1
    | EndTxnResponseV2
    | EndTxnResponseV3
    | EndTxnResponseV4
)



from kio.schema.api_versions.v0.request import (
    ApiVersionsRequest as ApiVersionsRequestV0,
)
from kio.schema.api_versions.v0.request import (
    RequestHeader as ApiVersionsRequestHeaderV0,
)
from kio.schema.api_versions.v0.response import (
    ApiVersionsResponse as ApiVersionsResponseV0,
)
from kio.schema.api_versions.v0.response import (
    ResponseHeader as ApiVersionsResponseHeaderV0,
)
from kio.schema.api_versions.v1.request import (
    ApiVersionsRequest as ApiVersionsRequestV1,
)
from kio.schema.api_versions.v1.request import (
    RequestHeader as ApiVersionsRequestHeaderV1,
)
from kio.schema.api_versions.v1.response import (
    ApiVersionsResponse as ApiVersionsResponseV1,
)
from kio.schema.api_versions.v1.response import (
    ResponseHeader as ApiVersionsResponseHeaderV1,
)
from kio.schema.api_versions.v2.request import (
    ApiVersionsRequest as ApiVersionsRequestV2,
)
from kio.schema.api_versions.v2.request import (
    RequestHeader as ApiVersionsRequestHeaderV2,
)
from kio.schema.api_versions.v2.response import (
    ApiVersionsResponse as ApiVersionsResponseV2,
)
from kio.schema.api_versions.v2.response import (
    ResponseHeader as ApiVersionsResponseHeaderV2,
)
from kio.schema.api_versions.v3.request import (
    ApiVersionsRequest as ApiVersionsRequestV3,
)
from kio.schema.api_versions.v3.request import (
    RequestHeader as ApiVersionsRequestHeaderV3,
)
from kio.schema.api_versions.v3.response import (
    ApiVersionsResponse as ApiVersionsResponseV3,
)
from kio.schema.api_versions.v3.response import (
    ResponseHeader as ApiVersionsResponseHeaderV3,
)
from kio.schema.api_versions.v4.request import (
    ApiVersionsRequest as ApiVersionsRequestV4,
)
from kio.schema.api_versions.v4.request import (
    RequestHeader as ApiVersionsRequestHeaderV4,
)
from kio.schema.api_versions.v4.response import (
    ApiVersionsResponse as ApiVersionsResponseV4,
)
from kio.schema.api_versions.v4.response import (
    ResponseHeader as ApiVersionsResponseHeaderV4,
)

ApiVersionsRequestHeader = (
    ApiVersionsRequestHeaderV0
    | ApiVersionsRequestHeaderV1
    | ApiVersionsRequestHeaderV2
    | ApiVersionsRequestHeaderV3
    | ApiVersionsRequestHeaderV4
)

ApiVersionsRequest = (
    ApiVersionsRequestV0
    | ApiVersionsRequestV1
    | ApiVersionsRequestV2
    | ApiVersionsRequestV3
    | ApiVersionsRequestV4
)

ApiVersionsResponse = (
    ApiVersionsResponseV0
    | ApiVersionsResponseV1
    | ApiVersionsResponseV2
    | ApiVersionsResponseV3
    | ApiVersionsResponseV4
)



from kio.schema.offset_delete.v0.request import (
    OffsetDeleteRequest as OffsetDeleteRequestV0,
)
from kio.schema.offset_delete.v0.request import (
    RequestHeader as OffsetDeleteRequestHeaderV0,
)
from kio.schema.offset_delete.v0.response import (
    OffsetDeleteResponse as OffsetDeleteResponseV0,
)
from kio.schema.offset_delete.v0.response import (
    ResponseHeader as OffsetDeleteResponseHeaderV0,
)

OffsetDeleteRequestHeader = OffsetDeleteRequestHeaderV0

OffsetDeleteResponseHeader = OffsetDeleteResponseHeaderV0

OffsetDeleteRequest = OffsetDeleteRequestV0

OffsetDeleteResponse = OffsetDeleteResponseV0



from kio.schema.stop_replica.v0.request import (
    StopReplicaRequest as StopReplicaRequestV0,
)
from kio.schema.stop_replica.v0.request import (
    RequestHeader as StopReplicaRequestHeaderV0,
)
from kio.schema.stop_replica.v0.response import (
    StopReplicaResponse as StopReplicaResponseV0,
)
from kio.schema.stop_replica.v0.response import (
    ResponseHeader as StopReplicaResponseHeaderV0,
)
from kio.schema.stop_replica.v1.request import (
    StopReplicaRequest as StopReplicaRequestV1,
)
from kio.schema.stop_replica.v1.request import (
    RequestHeader as StopReplicaRequestHeaderV1,
)
from kio.schema.stop_replica.v1.response import (
    StopReplicaResponse as StopReplicaResponseV1,
)
from kio.schema.stop_replica.v1.response import (
    ResponseHeader as StopReplicaResponseHeaderV1,
)
from kio.schema.stop_replica.v2.request import (
    StopReplicaRequest as StopReplicaRequestV2,
)
from kio.schema.stop_replica.v2.request import (
    RequestHeader as StopReplicaRequestHeaderV2,
)
from kio.schema.stop_replica.v2.response import (
    StopReplicaResponse as StopReplicaResponseV2,
)
from kio.schema.stop_replica.v2.response import (
    ResponseHeader as StopReplicaResponseHeaderV2,
)
from kio.schema.stop_replica.v3.request import (
    StopReplicaRequest as StopReplicaRequestV3,
)
from kio.schema.stop_replica.v3.request import (
    RequestHeader as StopReplicaRequestHeaderV3,
)
from kio.schema.stop_replica.v3.response import (
    StopReplicaResponse as StopReplicaResponseV3,
)
from kio.schema.stop_replica.v3.response import (
    ResponseHeader as StopReplicaResponseHeaderV3,
)
from kio.schema.stop_replica.v4.request import (
    StopReplicaRequest as StopReplicaRequestV4,
)
from kio.schema.stop_replica.v4.request import (
    RequestHeader as StopReplicaRequestHeaderV4,
)
from kio.schema.stop_replica.v4.response import (
    StopReplicaResponse as StopReplicaResponseV4,
)
from kio.schema.stop_replica.v4.response import (
    ResponseHeader as StopReplicaResponseHeaderV4,
)

StopReplicaRequestHeader = (
    StopReplicaRequestHeaderV0
    | StopReplicaRequestHeaderV1
    | StopReplicaRequestHeaderV2
    | StopReplicaRequestHeaderV3
    | StopReplicaRequestHeaderV4
)

StopReplicaResponseHeader = (
    StopReplicaResponseHeaderV0
    | StopReplicaResponseHeaderV1
    | StopReplicaResponseHeaderV2
    | StopReplicaResponseHeaderV3
    | StopReplicaResponseHeaderV4
)

StopReplicaRequest = (
    StopReplicaRequestV0
    | StopReplicaRequestV1
    | StopReplicaRequestV2
    | StopReplicaRequestV3
    | StopReplicaRequestV4
)

StopReplicaResponse = (
    StopReplicaResponseV0
    | StopReplicaResponseV1
    | StopReplicaResponseV2
    | StopReplicaResponseV3
    | StopReplicaResponseV4
)



from kio.schema.offset_fetch.v0.request import (
    OffsetFetchRequest as OffsetFetchRequestV0,
)
from kio.schema.offset_fetch.v0.request import (
    RequestHeader as OffsetFetchRequestHeaderV0,
)
from kio.schema.offset_fetch.v0.response import (
    OffsetFetchResponse as OffsetFetchResponseV0,
)
from kio.schema.offset_fetch.v0.response import (
    ResponseHeader as OffsetFetchResponseHeaderV0,
)
from kio.schema.offset_fetch.v1.request import (
    OffsetFetchRequest as OffsetFetchRequestV1,
)
from kio.schema.offset_fetch.v1.request import (
    RequestHeader as OffsetFetchRequestHeaderV1,
)
from kio.schema.offset_fetch.v1.response import (
    OffsetFetchResponse as OffsetFetchResponseV1,
)
from kio.schema.offset_fetch.v1.response import (
    ResponseHeader as OffsetFetchResponseHeaderV1,
)
from kio.schema.offset_fetch.v2.request import (
    OffsetFetchRequest as OffsetFetchRequestV2,
)
from kio.schema.offset_fetch.v2.request import (
    RequestHeader as OffsetFetchRequestHeaderV2,
)
from kio.schema.offset_fetch.v2.response import (
    OffsetFetchResponse as OffsetFetchResponseV2,
)
from kio.schema.offset_fetch.v2.response import (
    ResponseHeader as OffsetFetchResponseHeaderV2,
)
from kio.schema.offset_fetch.v3.request import (
    OffsetFetchRequest as OffsetFetchRequestV3,
)
from kio.schema.offset_fetch.v3.request import (
    RequestHeader as OffsetFetchRequestHeaderV3,
)
from kio.schema.offset_fetch.v3.response import (
    OffsetFetchResponse as OffsetFetchResponseV3,
)
from kio.schema.offset_fetch.v3.response import (
    ResponseHeader as OffsetFetchResponseHeaderV3,
)
from kio.schema.offset_fetch.v4.request import (
    OffsetFetchRequest as OffsetFetchRequestV4,
)
from kio.schema.offset_fetch.v4.request import (
    RequestHeader as OffsetFetchRequestHeaderV4,
)
from kio.schema.offset_fetch.v4.response import (
    OffsetFetchResponse as OffsetFetchResponseV4,
)
from kio.schema.offset_fetch.v4.response import (
    ResponseHeader as OffsetFetchResponseHeaderV4,
)
from kio.schema.offset_fetch.v5.request import (
    OffsetFetchRequest as OffsetFetchRequestV5,
)
from kio.schema.offset_fetch.v5.request import (
    RequestHeader as OffsetFetchRequestHeaderV5,
)
from kio.schema.offset_fetch.v5.response import (
    OffsetFetchResponse as OffsetFetchResponseV5,
)
from kio.schema.offset_fetch.v5.response import (
    ResponseHeader as OffsetFetchResponseHeaderV5,
)
from kio.schema.offset_fetch.v6.request import (
    OffsetFetchRequest as OffsetFetchRequestV6,
)
from kio.schema.offset_fetch.v6.request import (
    RequestHeader as OffsetFetchRequestHeaderV6,
)
from kio.schema.offset_fetch.v6.response import (
    OffsetFetchResponse as OffsetFetchResponseV6,
)
from kio.schema.offset_fetch.v6.response import (
    ResponseHeader as OffsetFetchResponseHeaderV6,
)
from kio.schema.offset_fetch.v7.request import (
    OffsetFetchRequest as OffsetFetchRequestV7,
)
from kio.schema.offset_fetch.v7.request import (
    RequestHeader as OffsetFetchRequestHeaderV7,
)
from kio.schema.offset_fetch.v7.response import (
    OffsetFetchResponse as OffsetFetchResponseV7,
)
from kio.schema.offset_fetch.v7.response import (
    ResponseHeader as OffsetFetchResponseHeaderV7,
)
from kio.schema.offset_fetch.v8.request import (
    OffsetFetchRequest as OffsetFetchRequestV8,
)
from kio.schema.offset_fetch.v8.request import (
    RequestHeader as OffsetFetchRequestHeaderV8,
)
from kio.schema.offset_fetch.v8.response import (
    OffsetFetchResponse as OffsetFetchResponseV8,
)
from kio.schema.offset_fetch.v8.response import (
    ResponseHeader as OffsetFetchResponseHeaderV8,
)
from kio.schema.offset_fetch.v9.request import (
    OffsetFetchRequest as OffsetFetchRequestV9,
)
from kio.schema.offset_fetch.v9.request import (
    RequestHeader as OffsetFetchRequestHeaderV9,
)
from kio.schema.offset_fetch.v9.response import (
    OffsetFetchResponse as OffsetFetchResponseV9,
)
from kio.schema.offset_fetch.v9.response import (
    ResponseHeader as OffsetFetchResponseHeaderV9,
)


OffsetFetchRequestHeader = (
    OffsetFetchRequestHeaderV0
    | OffsetFetchRequestHeaderV1
    | OffsetFetchRequestHeaderV2
    | OffsetFetchRequestHeaderV3
    | OffsetFetchRequestHeaderV4
    | OffsetFetchRequestHeaderV5
    | OffsetFetchRequestHeaderV6
    | OffsetFetchRequestHeaderV7
    | OffsetFetchRequestHeaderV8
    | OffsetFetchRequestHeaderV9
)

OffsetFetchResponseHeader = (
    OffsetFetchResponseHeaderV0
    | OffsetFetchResponseHeaderV1
    | OffsetFetchResponseHeaderV2
    | OffsetFetchResponseHeaderV3
    | OffsetFetchResponseHeaderV4
    | OffsetFetchResponseHeaderV5
    | OffsetFetchResponseHeaderV6
    | OffsetFetchResponseHeaderV7
    | OffsetFetchResponseHeaderV8
    | OffsetFetchResponseHeaderV9
)

OffsetFetchRequest = (
    OffsetFetchRequestV0
    | OffsetFetchRequestV1
    | OffsetFetchRequestV2
    | OffsetFetchRequestV3
    | OffsetFetchRequestV4
    | OffsetFetchRequestV5
    | OffsetFetchRequestV6
    | OffsetFetchRequestV7
    | OffsetFetchRequestV8
    | OffsetFetchRequestV9
)

OffsetFetchResponse = (
    OffsetFetchResponseV0
    | OffsetFetchResponseV1
    | OffsetFetchResponseV2
    | OffsetFetchResponseV3
    | OffsetFetchResponseV4
    | OffsetFetchResponseV5
    | OffsetFetchResponseV6
    | OffsetFetchResponseV7
    | OffsetFetchResponseV8
    | OffsetFetchResponseV9
)



import datetime

from kio.static.primitive import i32Timedelta

from icestream.config import Config


def wal_uri_to_object_key(config: Config, uri: str) -> str:
    bucket = config.WAL_BUCKET
    prefix = (config.WAL_BUCKET_PREFIX or "").strip("/")
    # strip bucket from start
    if uri.startswith(bucket + "/"):
        key = uri[len(bucket) + 1:]
    else:
        key = uri
    # strip prefix if set
    if prefix and key.startswith(prefix + "/"):
        key = key[len(prefix) + 1:]
    return key

def zero_throttle():
    return i32Timedelta.parse(datetime.timedelta(milliseconds=0))



import struct
import time
from io import BytesIO
from typing import List

from icestream.kafkaserver.protocol import (
    KafkaRecordBatch,
    KafkaRecordHeader,
    KafkaRecord, decode_kafka_records,
)
from icestream.kafkaserver.types import ProduceTopicPartitionData
from icestream.kafkaserver.utils import encode_varint, decode_varint
from icestream.kafkaserver.wal import WALBatch, WALFile


def encode_kafka_wal_file_with_offsets(
        batches: List[ProduceTopicPartitionData], broker_id: str
) -> tuple[bytes, list[dict]]:
    buf = BytesIO()
    offset_metadata = []

    buf.write(b"WAL1")
    buf.write(struct.pack(">B", 1))
    buf.write(struct.pack(">Q", int(time.time() * 1000)))
    broker_bytes = broker_id.encode("utf-8")
    buf.write(encode_varint(len(broker_bytes)))
    buf.write(broker_bytes)
    buf.write(encode_varint(len(batches)))

    for batch in batches:
        topic_bytes = batch.topic.encode("utf-8")
        buf.write(encode_varint(len(topic_bytes)))
        buf.write(topic_bytes)
        buf.write(struct.pack(">i", batch.partition))

        rb = batch.kafka_record_batch

        record_batch_bytes = rb.to_bytes()

        byte_start = buf.tell()
        buf.write(encode_varint(len(record_batch_bytes)))
        buf.write(record_batch_bytes)
        byte_end = buf.tell()

        now_ts = int(time.time() * 1000)
        min_ts = now_ts
        max_ts = now_ts

        if rb.base_timestamp != 0 and rb.max_timestamp != 0:
            min_ts = rb.base_timestamp
            max_ts = rb.max_timestamp

        offset_metadata.append(
            {
                "topic": batch.topic,
                "partition": batch.partition,
                "base_offset": rb.base_offset,
                "last_offset": rb.base_offset + rb.last_offset_delta,
                "byte_start": byte_start,
                "byte_end": byte_end,
                "min_timestamp": min_ts,
                "max_timestamp": max_ts,
            }
        )

    return buf.getvalue(), offset_metadata


def decode_kafka_wal_file(data: bytes) -> WALFile:
    buf = BytesIO(data)

    if buf.read(4) != b"WAL1":
        raise ValueError("Invalid WAL file magic")

    version = struct.unpack(">B", buf.read(1))[0]
    flushed_at = struct.unpack(">Q", buf.read(8))[0]
    broker_id_len = decode_varint(buf)
    broker_id = buf.read(broker_id_len).decode("utf-8")
    batch_count = decode_varint(buf)

    batches = []
    for _ in range(batch_count):
        topic_len = decode_varint(buf)
        topic = buf.read(topic_len).decode("utf-8")
        partition = struct.unpack(">i", buf.read(4))[0]
        batch_len = decode_varint(buf)
        record_batch_bytes = buf.read(batch_len)
        kafka_record_batch = KafkaRecordBatch.from_bytes(record_batch_bytes)
        batches.append(WALBatch(topic, partition, kafka_record_batch))

    return WALFile(version, flushed_at, broker_id, batches)



from kio.schema.delete_acls.v0.request import (
    DeleteAclsRequest as DeleteAclsRequestV0,
)
from kio.schema.delete_acls.v0.request import (
    RequestHeader as DeleteAclsRequestHeaderV0,
)
from kio.schema.delete_acls.v0.response import (
    DeleteAclsResponse as DeleteAclsResponseV0,
)
from kio.schema.delete_acls.v0.response import (
    ResponseHeader as DeleteAclsResponseHeaderV0,
)
from kio.schema.delete_acls.v1.request import (
    DeleteAclsRequest as DeleteAclsRequestV1,
)
from kio.schema.delete_acls.v1.request import (
    RequestHeader as DeleteAclsRequestHeaderV1,
)
from kio.schema.delete_acls.v1.response import (
    DeleteAclsResponse as DeleteAclsResponseV1,
)
from kio.schema.delete_acls.v1.response import (
    ResponseHeader as DeleteAclsResponseHeaderV1,
)
from kio.schema.delete_acls.v2.request import (
    DeleteAclsRequest as DeleteAclsRequestV2,
)
from kio.schema.delete_acls.v2.request import (
    RequestHeader as DeleteAclsRequestHeaderV2,
)
from kio.schema.delete_acls.v2.response import (
    DeleteAclsResponse as DeleteAclsResponseV2,
)
from kio.schema.delete_acls.v2.response import (
    ResponseHeader as DeleteAclsResponseHeaderV2,
)
from kio.schema.delete_acls.v3.request import (
    DeleteAclsRequest as DeleteAclsRequestV3,
)
from kio.schema.delete_acls.v3.request import (
    RequestHeader as DeleteAclsRequestHeaderV3,
)
from kio.schema.delete_acls.v3.response import (
    DeleteAclsResponse as DeleteAclsResponseV3,
)
from kio.schema.delete_acls.v3.response import (
    ResponseHeader as DeleteAclsResponseHeaderV3,
)

DeleteAclsRequestHeader = (
    DeleteAclsRequestHeaderV0
    | DeleteAclsRequestHeaderV1
    | DeleteAclsRequestHeaderV2
    | DeleteAclsRequestHeaderV3
)

DeleteAclsResponseHeader = (
    DeleteAclsResponseHeaderV0
    | DeleteAclsResponseHeaderV1
    | DeleteAclsResponseHeaderV2
    | DeleteAclsResponseHeaderV3
)

DeleteAclsRequest = (
    DeleteAclsRequestV0
    | DeleteAclsRequestV1
    | DeleteAclsRequestV2
    | DeleteAclsRequestV3
)

DeleteAclsResponse = (
    DeleteAclsResponseV0
    | DeleteAclsResponseV1
    | DeleteAclsResponseV2
    | DeleteAclsResponseV3
)



from kio.schema.update_features.v0.request import (
    UpdateFeaturesRequest as UpdateFeaturesRequestV0,
)
from kio.schema.update_features.v0.request import (
    RequestHeader as UpdateFeaturesRequestHeaderV0,
)
from kio.schema.update_features.v0.response import (
    UpdateFeaturesResponse as UpdateFeaturesResponseV0,
)
from kio.schema.update_features.v0.response import (
    ResponseHeader as UpdateFeaturesResponseHeaderV0,
)
from kio.schema.update_features.v1.request import (
    UpdateFeaturesRequest as UpdateFeaturesRequestV1,
)
from kio.schema.update_features.v1.request import (
    RequestHeader as UpdateFeaturesRequestHeaderV1,
)
from kio.schema.update_features.v1.response import (
    UpdateFeaturesResponse as UpdateFeaturesResponseV1,
)
from kio.schema.update_features.v1.response import (
    ResponseHeader as UpdateFeaturesResponseHeaderV1,
)

UpdateFeaturesRequestHeader = (
    UpdateFeaturesRequestHeaderV0 | UpdateFeaturesRequestHeaderV1
)
UpdateFeaturesResponseHeader = (
    UpdateFeaturesResponseHeaderV0 | UpdateFeaturesResponseHeaderV1
)
UpdateFeaturesRequest = UpdateFeaturesRequestV0 | UpdateFeaturesRequestV1
UpdateFeaturesResponse = UpdateFeaturesResponseV0 | UpdateFeaturesResponseV1



from kio.schema.describe_transactions.v0.request import (
    DescribeTransactionsRequest as DescribeTransactionsRequestV0,
)
from kio.schema.describe_transactions.v0.request import (
    RequestHeader as DescribeTransactionsRequestHeaderV0,
)
from kio.schema.describe_transactions.v0.response import (
    DescribeTransactionsResponse as DescribeTransactionsResponseV0,
)
from kio.schema.describe_transactions.v0.response import (
    ResponseHeader as DescribeTransactionsResponseHeaderV0,
)

DescribeTransactionsRequestHeader = DescribeTransactionsRequestHeaderV0
DescribeTransactionsResponseHeader = DescribeTransactionsResponseHeaderV0
DescribeTransactionsRequest = DescribeTransactionsRequestV0
DescribeTransactionsResponse = DescribeTransactionsResponseV0



from kio.schema.alter_configs.v0.request import (
    AlterConfigsRequest as AlterConfigsRequestV0,
)
from kio.schema.alter_configs.v0.request import (
    RequestHeader as AlterConfigsRequestHeaderV0,
)
from kio.schema.alter_configs.v0.response import (
    AlterConfigsResponse as AlterConfigsResponseV0,
)
from kio.schema.alter_configs.v0.response import (
    ResponseHeader as AlterConfigsResponseHeaderV0,
)
from kio.schema.alter_configs.v1.request import (
    AlterConfigsRequest as AlterConfigsRequestV1,
)
from kio.schema.alter_configs.v1.request import (
    RequestHeader as AlterConfigsRequestHeaderV1,
)
from kio.schema.alter_configs.v1.response import (
    AlterConfigsResponse as AlterConfigsResponseV1,
)
from kio.schema.alter_configs.v1.response import (
    ResponseHeader as AlterConfigsResponseHeaderV1,
)
from kio.schema.alter_configs.v2.request import (
    AlterConfigsRequest as AlterConfigsRequestV2,
)
from kio.schema.alter_configs.v2.request import (
    RequestHeader as AlterConfigsRequestHeaderV2,
)
from kio.schema.alter_configs.v2.response import (
    AlterConfigsResponse as AlterConfigsResponseV2,
)
from kio.schema.alter_configs.v2.response import (
    ResponseHeader as AlterConfigsResponseHeaderV2,
)

AlterConfigsRequestHeader = (
    AlterConfigsRequestHeaderV0 | AlterConfigsRequestHeaderV1 | AlterConfigsRequestHeaderV2
)

AlterConfigsResponseHeader = (
    AlterConfigsResponseHeaderV0 | AlterConfigsResponseHeaderV1 | AlterConfigsResponseHeaderV2
)

AlterConfigsRequest = (
    AlterConfigsRequestV0 | AlterConfigsRequestV1 | AlterConfigsRequestV2
)

AlterConfigsResponse = (
    AlterConfigsResponseV0 | AlterConfigsResponseV1 | AlterConfigsResponseV2
)



from kio.schema.describe_log_dirs.v0.request import (
    DescribeLogDirsRequest as DescribeLogDirsRequestV0,
)
from kio.schema.describe_log_dirs.v0.request import (
    RequestHeader as DescribeLogDirsRequestHeaderV0,
)
from kio.schema.describe_log_dirs.v0.response import (
    DescribeLogDirsResponse as DescribeLogDirsResponseV0,
)
from kio.schema.describe_log_dirs.v0.response import (
    ResponseHeader as DescribeLogDirsResponseHeaderV0,
)
from kio.schema.describe_log_dirs.v1.request import (
    DescribeLogDirsRequest as DescribeLogDirsRequestV1,
)
from kio.schema.describe_log_dirs.v1.request import (
    RequestHeader as DescribeLogDirsRequestHeaderV1,
)
from kio.schema.describe_log_dirs.v1.response import (
    DescribeLogDirsResponse as DescribeLogDirsResponseV1,
)
from kio.schema.describe_log_dirs.v1.response import (
    ResponseHeader as DescribeLogDirsResponseHeaderV1,
)
from kio.schema.describe_log_dirs.v2.request import (
    DescribeLogDirsRequest as DescribeLogDirsRequestV2,
)
from kio.schema.describe_log_dirs.v2.request import (
    RequestHeader as DescribeLogDirsRequestHeaderV2,
)
from kio.schema.describe_log_dirs.v2.response import (
    DescribeLogDirsResponse as DescribeLogDirsResponseV2,
)
from kio.schema.describe_log_dirs.v2.response import (
    ResponseHeader as DescribeLogDirsResponseHeaderV2,
)
from kio.schema.describe_log_dirs.v3.request import (
    DescribeLogDirsRequest as DescribeLogDirsRequestV3,
)
from kio.schema.describe_log_dirs.v3.request import (
    RequestHeader as DescribeLogDirsRequestHeaderV3,
)
from kio.schema.describe_log_dirs.v3.response import (
    DescribeLogDirsResponse as DescribeLogDirsResponseV3,
)
from kio.schema.describe_log_dirs.v3.response import (
    ResponseHeader as DescribeLogDirsResponseHeaderV3,
)
from kio.schema.describe_log_dirs.v4.request import (
    DescribeLogDirsRequest as DescribeLogDirsRequestV4,
)
from kio.schema.describe_log_dirs.v4.request import (
    RequestHeader as DescribeLogDirsRequestHeaderV4,
)
from kio.schema.describe_log_dirs.v4.response import (
    DescribeLogDirsResponse as DescribeLogDirsResponseV4,
)
from kio.schema.describe_log_dirs.v4.response import (
    ResponseHeader as DescribeLogDirsResponseHeaderV4,
)


DescribeLogDirsRequestHeader = (
    DescribeLogDirsRequestHeaderV0
    | DescribeLogDirsRequestHeaderV1
    | DescribeLogDirsRequestHeaderV2
    | DescribeLogDirsRequestHeaderV3
    | DescribeLogDirsRequestHeaderV4
)

DescribeLogDirsResponseHeader = (
    DescribeLogDirsResponseHeaderV0
    | DescribeLogDirsResponseHeaderV1
    | DescribeLogDirsResponseHeaderV2
    | DescribeLogDirsResponseHeaderV3
    | DescribeLogDirsResponseHeaderV4
)

DescribeLogDirsRequest = (
    DescribeLogDirsRequestV0
    | DescribeLogDirsRequestV1
    | DescribeLogDirsRequestV2
    | DescribeLogDirsRequestV3
    | DescribeLogDirsRequestV4
)

DescribeLogDirsResponse = (
    DescribeLogDirsResponseV0
    | DescribeLogDirsResponseV1
    | DescribeLogDirsResponseV2
    | DescribeLogDirsResponseV3
    | DescribeLogDirsResponseV4
)



from kio.schema.add_raft_voter.v0.request import (
    AddRaftVoterRequest as AddRaftVoterRequestV0,
)
from kio.schema.add_raft_voter.v0.request import (
    RequestHeader as AddRaftVoterRequestHeaderV0,
)
from kio.schema.add_raft_voter.v0.response import (
    AddRaftVoterResponse as AddRaftVoterResponseV0,
)
from kio.schema.add_raft_voter.v0.response import (
    ResponseHeader as AddRaftVoterResponseHeaderV0,
)

AddRaftVoterRequestHeader = AddRaftVoterRequestHeaderV0
AddRaftVoterResponseHeader = AddRaftVoterResponseHeaderV0
AddRaftVoterRequest = AddRaftVoterRequestV0
AddRaftVoterResponse = AddRaftVoterResponseV0



from kio.schema.delete_topics.v0.request import (
    DeleteTopicsRequest as DeleteTopicsRequestV0,
)
from kio.schema.delete_topics.v0.request import (
    RequestHeader as DeleteTopicsRequestHeaderV0,
)
from kio.schema.delete_topics.v0.response import (
    DeleteTopicsResponse as DeleteTopicsResponseV0,
)
from kio.schema.delete_topics.v0.response import (
    ResponseHeader as DeleteTopicsResponseHeaderV0,
)
from kio.schema.delete_topics.v1.request import (
    DeleteTopicsRequest as DeleteTopicsRequestV1,
)
from kio.schema.delete_topics.v1.request import (
    RequestHeader as DeleteTopicsRequestHeaderV1,
)
from kio.schema.delete_topics.v1.response import (
    DeleteTopicsResponse as DeleteTopicsResponseV1,
)
from kio.schema.delete_topics.v1.response import (
    ResponseHeader as DeleteTopicsResponseHeaderV1,
)
from kio.schema.delete_topics.v2.request import (
    DeleteTopicsRequest as DeleteTopicsRequestV2,
)
from kio.schema.delete_topics.v2.request import (
    RequestHeader as DeleteTopicsRequestHeaderV2,
)
from kio.schema.delete_topics.v2.response import (
    DeleteTopicsResponse as DeleteTopicsResponseV2,
)
from kio.schema.delete_topics.v2.response import (
    ResponseHeader as DeleteTopicsResponseHeaderV2,
)
from kio.schema.delete_topics.v3.request import (
    DeleteTopicsRequest as DeleteTopicsRequestV3,
)
from kio.schema.delete_topics.v3.request import (
    RequestHeader as DeleteTopicsRequestHeaderV3,
)
from kio.schema.delete_topics.v3.response import (
    DeleteTopicsResponse as DeleteTopicsResponseV3,
)
from kio.schema.delete_topics.v3.response import (
    ResponseHeader as DeleteTopicsResponseHeaderV3,
)
from kio.schema.delete_topics.v4.request import (
    DeleteTopicsRequest as DeleteTopicsRequestV4,
)
from kio.schema.delete_topics.v4.request import (
    RequestHeader as DeleteTopicsRequestHeaderV4,
)
from kio.schema.delete_topics.v4.response import (
    DeleteTopicsResponse as DeleteTopicsResponseV4,
)
from kio.schema.delete_topics.v4.response import (
    ResponseHeader as DeleteTopicsResponseHeaderV4,
)
from kio.schema.delete_topics.v5.request import (
    DeleteTopicsRequest as DeleteTopicsRequestV5,
)
from kio.schema.delete_topics.v5.request import (
    RequestHeader as DeleteTopicsRequestHeaderV5,
)
from kio.schema.delete_topics.v5.response import (
    DeleteTopicsResponse as DeleteTopicsResponseV5,
)
from kio.schema.delete_topics.v5.response import (
    ResponseHeader as DeleteTopicsResponseHeaderV5,
)
from kio.schema.delete_topics.v6.request import (
    DeleteTopicsRequest as DeleteTopicsRequestV6,
)
from kio.schema.delete_topics.v6.request import (
    RequestHeader as DeleteTopicsRequestHeaderV6,
)
from kio.schema.delete_topics.v6.response import (
    DeleteTopicsResponse as DeleteTopicsResponseV6,
)
from kio.schema.delete_topics.v6.response import (
    ResponseHeader as DeleteTopicsResponseHeaderV6,
)

DeleteTopicsRequestHeader = (
    DeleteTopicsRequestHeaderV0
    | DeleteTopicsRequestHeaderV1
    | DeleteTopicsRequestHeaderV2
    | DeleteTopicsRequestHeaderV3
    | DeleteTopicsRequestHeaderV4
    | DeleteTopicsRequestHeaderV5
    | DeleteTopicsRequestHeaderV6
)

DeleteTopicsResponseHeader = (
    DeleteTopicsResponseHeaderV0
    | DeleteTopicsResponseHeaderV1
    | DeleteTopicsResponseHeaderV2
    | DeleteTopicsResponseHeaderV3
    | DeleteTopicsResponseHeaderV4
    | DeleteTopicsResponseHeaderV5
    | DeleteTopicsResponseHeaderV6
)

DeleteTopicsRequest = (
    DeleteTopicsRequestV0
    | DeleteTopicsRequestV1
    | DeleteTopicsRequestV2
    | DeleteTopicsRequestV3
    | DeleteTopicsRequestV4
    | DeleteTopicsRequestV5
    | DeleteTopicsRequestV6
)

DeleteTopicsResponse = (
    DeleteTopicsResponseV0
    | DeleteTopicsResponseV1
    | DeleteTopicsResponseV2
    | DeleteTopicsResponseV3
    | DeleteTopicsResponseV4
    | DeleteTopicsResponseV5
    | DeleteTopicsResponseV6
)



from kio.schema.txn_offset_commit.v0.request import (
    TxnOffsetCommitRequest as TxnOffsetCommitRequestV0,
)
from kio.schema.txn_offset_commit.v0.request import (
    RequestHeader as TxnOffsetCommitRequestHeaderV0,
)
from kio.schema.txn_offset_commit.v0.response import (
    TxnOffsetCommitResponse as TxnOffsetCommitResponseV0,
)
from kio.schema.txn_offset_commit.v0.response import (
    ResponseHeader as TxnOffsetCommitResponseHeaderV0,
)
from kio.schema.txn_offset_commit.v1.request import (
    TxnOffsetCommitRequest as TxnOffsetCommitRequestV1,
)
from kio.schema.txn_offset_commit.v1.request import (
    RequestHeader as TxnOffsetCommitRequestHeaderV1,
)
from kio.schema.txn_offset_commit.v1.response import (
    TxnOffsetCommitResponse as TxnOffsetCommitResponseV1,
)
from kio.schema.txn_offset_commit.v1.response import (
    ResponseHeader as TxnOffsetCommitResponseHeaderV1,
)
from kio.schema.txn_offset_commit.v2.request import (
    TxnOffsetCommitRequest as TxnOffsetCommitRequestV2,
)
from kio.schema.txn_offset_commit.v2.request import (
    RequestHeader as TxnOffsetCommitRequestHeaderV2,
)
from kio.schema.txn_offset_commit.v2.response import (
    TxnOffsetCommitResponse as TxnOffsetCommitResponseV2,
)
from kio.schema.txn_offset_commit.v2.response import (
    ResponseHeader as TxnOffsetCommitResponseHeaderV2,
)
from kio.schema.txn_offset_commit.v3.request import (
    TxnOffsetCommitRequest as TxnOffsetCommitRequestV3,
)
from kio.schema.txn_offset_commit.v3.request import (
    RequestHeader as TxnOffsetCommitRequestHeaderV3,
)
from kio.schema.txn_offset_commit.v3.response import (
    TxnOffsetCommitResponse as TxnOffsetCommitResponseV3,
)
from kio.schema.txn_offset_commit.v3.response import (
    ResponseHeader as TxnOffsetCommitResponseHeaderV3,
)
from kio.schema.txn_offset_commit.v4.request import (
    TxnOffsetCommitRequest as TxnOffsetCommitRequestV4,
)
from kio.schema.txn_offset_commit.v4.request import (
    RequestHeader as TxnOffsetCommitRequestHeaderV4,
)
from kio.schema.txn_offset_commit.v4.response import (
    TxnOffsetCommitResponse as TxnOffsetCommitResponseV4,
)
from kio.schema.txn_offset_commit.v4.response import (
    ResponseHeader as TxnOffsetCommitResponseHeaderV4,
)

TxnOffsetCommitRequestHeader = (
    TxnOffsetCommitRequestHeaderV0
    | TxnOffsetCommitRequestHeaderV1
    | TxnOffsetCommitRequestHeaderV2
    | TxnOffsetCommitRequestHeaderV3
    | TxnOffsetCommitRequestHeaderV4
)

TxnOffsetCommitResponseHeader = (
    TxnOffsetCommitResponseHeaderV0
    | TxnOffsetCommitResponseHeaderV1
    | TxnOffsetCommitResponseHeaderV2
    | TxnOffsetCommitResponseHeaderV3
    | TxnOffsetCommitResponseHeaderV4
)

TxnOffsetCommitRequest = (
    TxnOffsetCommitRequestV0
    | TxnOffsetCommitRequestV1
    | TxnOffsetCommitRequestV2
    | TxnOffsetCommitRequestV3
    | TxnOffsetCommitRequestV4
)

TxnOffsetCommitResponse = (
    TxnOffsetCommitResponseV0
    | TxnOffsetCommitResponseV1
    | TxnOffsetCommitResponseV2
    | TxnOffsetCommitResponseV3
    | TxnOffsetCommitResponseV4
)



from kio.schema.write_share_group_state.v0.request import (
    WriteShareGroupStateRequest as WriteShareGroupStateRequestV0,
)
from kio.schema.write_share_group_state.v0.request import (
    RequestHeader as WriteShareGroupStateRequestHeaderV0,
)
from kio.schema.write_share_group_state.v0.response import (
    WriteShareGroupStateResponse as WriteShareGroupStateResponseV0,
)
from kio.schema.write_share_group_state.v0.response import (
    ResponseHeader as WriteShareGroupStateResponseHeaderV0,
)

WriteShareGroupStateRequestHeader = WriteShareGroupStateRequestHeaderV0
WriteShareGroupStateResponseHeader = WriteShareGroupStateResponseHeaderV0
WriteShareGroupStateRequest = WriteShareGroupStateRequestV0
WriteShareGroupStateResponse = WriteShareGroupStateResponseV0



from kio.schema.create_delegation_token.v0.request import CreateDelegationTokenRequest as CreateDelegationTokenRequestV0
from kio.schema.create_delegation_token.v0.request import RequestHeader as CreateDelegationTokenRequestHeaderV0
from kio.schema.create_delegation_token.v0.response import CreateDelegationTokenResponse as CreateDelegationTokenResponseV0
from kio.schema.create_delegation_token.v0.response import ResponseHeader as CreateDelegationTokenResponseHeaderV0
from kio.schema.create_delegation_token.v1.request import CreateDelegationTokenRequest as CreateDelegationTokenRequestV1
from kio.schema.create_delegation_token.v1.request import RequestHeader as CreateDelegationTokenRequestHeaderV1
from kio.schema.create_delegation_token.v1.response import CreateDelegationTokenResponse as CreateDelegationTokenResponseV1
from kio.schema.create_delegation_token.v1.response import ResponseHeader as CreateDelegationTokenResponseHeaderV1
from kio.schema.create_delegation_token.v2.request import CreateDelegationTokenRequest as CreateDelegationTokenRequestV2
from kio.schema.create_delegation_token.v2.request import RequestHeader as CreateDelegationTokenRequestHeaderV2
from kio.schema.create_delegation_token.v2.response import CreateDelegationTokenResponse as CreateDelegationTokenResponseV2
from kio.schema.create_delegation_token.v2.response import ResponseHeader as CreateDelegationTokenResponseHeaderV2
from kio.schema.create_delegation_token.v3.request import CreateDelegationTokenRequest as CreateDelegationTokenRequestV3
from kio.schema.create_delegation_token.v3.request import RequestHeader as CreateDelegationTokenRequestHeaderV3
from kio.schema.create_delegation_token.v3.response import CreateDelegationTokenResponse as CreateDelegationTokenResponseV3
from kio.schema.create_delegation_token.v3.response import ResponseHeader as CreateDelegationTokenResponseHeaderV3

CreateDelegationTokenRequestHeader = (
    CreateDelegationTokenRequestHeaderV0
    | CreateDelegationTokenRequestHeaderV1
    | CreateDelegationTokenRequestHeaderV2
    | CreateDelegationTokenRequestHeaderV3
)

CreateDelegationTokenResponseHeader = (
    CreateDelegationTokenResponseHeaderV0
    | CreateDelegationTokenResponseHeaderV1
    | CreateDelegationTokenResponseHeaderV2
    | CreateDelegationTokenResponseHeaderV3
)

CreateDelegationTokenRequest = (
    CreateDelegationTokenRequestV0
    | CreateDelegationTokenRequestV1
    | CreateDelegationTokenRequestV2
    | CreateDelegationTokenRequestV3
)

CreateDelegationTokenResponse = (
    CreateDelegationTokenResponseV0
    | CreateDelegationTokenResponseV1
    | CreateDelegationTokenResponseV2
    | CreateDelegationTokenResponseV3
)



from kio.schema.remove_raft_voter.v0.request import (
    RemoveRaftVoterRequest as RemoveRaftVoterRequestV0,
)
from kio.schema.remove_raft_voter.v0.request import (
    RequestHeader as RemoveRaftVoterRequestHeaderV0,
)
from kio.schema.remove_raft_voter.v0.response import (
    RemoveRaftVoterResponse as RemoveRaftVoterResponseV0,
)
from kio.schema.remove_raft_voter.v0.response import (
    ResponseHeader as RemoveRaftVoterResponseHeaderV0,
)

RemoveRaftVoterRequestHeader = RemoveRaftVoterRequestHeaderV0

RemoveRaftVoterResponseHeader = RemoveRaftVoterResponseHeaderV0

RemoveRaftVoterRequest = RemoveRaftVoterRequestV0

RemoveRaftVoterResponse = RemoveRaftVoterResponseV0



from kio.schema.offset_commit.v0.request import (
    OffsetCommitRequest as OffsetCommitRequestV0,
)
from kio.schema.offset_commit.v0.request import (
    RequestHeader as OffsetCommitRequestHeaderV0,
)
from kio.schema.offset_commit.v0.response import (
    OffsetCommitResponse as OffsetCommitResponseV0,
)
from kio.schema.offset_commit.v0.response import (
    ResponseHeader as OffsetCommitResponseHeaderV0,
)
from kio.schema.offset_commit.v1.request import (
    OffsetCommitRequest as OffsetCommitRequestV1,
)
from kio.schema.offset_commit.v1.request import (
    RequestHeader as OffsetCommitRequestHeaderV1,
)
from kio.schema.offset_commit.v1.response import (
    OffsetCommitResponse as OffsetCommitResponseV1,
)
from kio.schema.offset_commit.v1.response import (
    ResponseHeader as OffsetCommitResponseHeaderV1,
)
from kio.schema.offset_commit.v2.request import (
    OffsetCommitRequest as OffsetCommitRequestV2,
)
from kio.schema.offset_commit.v2.request import (
    RequestHeader as OffsetCommitRequestHeaderV2,
)
from kio.schema.offset_commit.v2.response import (
    OffsetCommitResponse as OffsetCommitResponseV2,
)
from kio.schema.offset_commit.v2.response import (
    ResponseHeader as OffsetCommitResponseHeaderV2,
)
from kio.schema.offset_commit.v3.request import (
    OffsetCommitRequest as OffsetCommitRequestV3,
)
from kio.schema.offset_commit.v3.request import (
    RequestHeader as OffsetCommitRequestHeaderV3,
)
from kio.schema.offset_commit.v3.response import (
    OffsetCommitResponse as OffsetCommitResponseV3,
)
from kio.schema.offset_commit.v3.response import (
    ResponseHeader as OffsetCommitResponseHeaderV3,
)
from kio.schema.offset_commit.v4.request import (
    OffsetCommitRequest as OffsetCommitRequestV4,
)
from kio.schema.offset_commit.v4.request import (
    RequestHeader as OffsetCommitRequestHeaderV4,
)
from kio.schema.offset_commit.v4.response import (
    OffsetCommitResponse as OffsetCommitResponseV4,
)
from kio.schema.offset_commit.v4.response import (
    ResponseHeader as OffsetCommitResponseHeaderV4,
)
from kio.schema.offset_commit.v5.request import (
    OffsetCommitRequest as OffsetCommitRequestV5,
)
from kio.schema.offset_commit.v5.request import (
    RequestHeader as OffsetCommitRequestHeaderV5,
)
from kio.schema.offset_commit.v5.response import (
    OffsetCommitResponse as OffsetCommitResponseV5,
)
from kio.schema.offset_commit.v5.response import (
    ResponseHeader as OffsetCommitResponseHeaderV5,
)
from kio.schema.offset_commit.v6.request import (
    OffsetCommitRequest as OffsetCommitRequestV6,
)
from kio.schema.offset_commit.v6.request import (
    RequestHeader as OffsetCommitRequestHeaderV6,
)
from kio.schema.offset_commit.v6.response import (
    OffsetCommitResponse as OffsetCommitResponseV6,
)
from kio.schema.offset_commit.v6.response import (
    ResponseHeader as OffsetCommitResponseHeaderV6,
)
from kio.schema.offset_commit.v7.request import (
    OffsetCommitRequest as OffsetCommitRequestV7,
)
from kio.schema.offset_commit.v7.request import (
    RequestHeader as OffsetCommitRequestHeaderV7,
)
from kio.schema.offset_commit.v7.response import (
    OffsetCommitResponse as OffsetCommitResponseV7,
)
from kio.schema.offset_commit.v7.response import (
    ResponseHeader as OffsetCommitResponseHeaderV7,
)
from kio.schema.offset_commit.v8.request import (
    OffsetCommitRequest as OffsetCommitRequestV8,
)
from kio.schema.offset_commit.v8.request import (
    RequestHeader as OffsetCommitRequestHeaderV8,
)
from kio.schema.offset_commit.v8.response import (
    OffsetCommitResponse as OffsetCommitResponseV8,
)
from kio.schema.offset_commit.v8.response import (
    ResponseHeader as OffsetCommitResponseHeaderV8,
)
from kio.schema.offset_commit.v9.request import (
    OffsetCommitRequest as OffsetCommitRequestV9,
)
from kio.schema.offset_commit.v9.request import (
    RequestHeader as OffsetCommitRequestHeaderV9,
)
from kio.schema.offset_commit.v9.response import (
    OffsetCommitResponse as OffsetCommitResponseV9,
)
from kio.schema.offset_commit.v9.response import (
    ResponseHeader as OffsetCommitResponseHeaderV9,
)


OffsetCommitRequestHeader = (
    OffsetCommitRequestHeaderV0
    | OffsetCommitRequestHeaderV1
    | OffsetCommitRequestHeaderV2
    | OffsetCommitRequestHeaderV3
    | OffsetCommitRequestHeaderV4
    | OffsetCommitRequestHeaderV5
    | OffsetCommitRequestHeaderV6
    | OffsetCommitRequestHeaderV7
    | OffsetCommitRequestHeaderV8
    | OffsetCommitRequestHeaderV9
)

OffsetCommitResponseHeader = (
    OffsetCommitResponseHeaderV0
    | OffsetCommitResponseHeaderV1
    | OffsetCommitResponseHeaderV2
    | OffsetCommitResponseHeaderV3
    | OffsetCommitResponseHeaderV4
    | OffsetCommitResponseHeaderV5
    | OffsetCommitResponseHeaderV6
    | OffsetCommitResponseHeaderV7
    | OffsetCommitResponseHeaderV8
    | OffsetCommitResponseHeaderV9
)

OffsetCommitRequest = (
    OffsetCommitRequestV0
    | OffsetCommitRequestV1
    | OffsetCommitRequestV2
    | OffsetCommitRequestV3
    | OffsetCommitRequestV4
    | OffsetCommitRequestV5
    | OffsetCommitRequestV6
    | OffsetCommitRequestV7
    | OffsetCommitRequestV8
    | OffsetCommitRequestV9
)

OffsetCommitResponse = (
    OffsetCommitResponseV0
    | OffsetCommitResponseV1
    | OffsetCommitResponseV2
    | OffsetCommitResponseV3
    | OffsetCommitResponseV4
    | OffsetCommitResponseV5
    | OffsetCommitResponseV6
    | OffsetCommitResponseV7
    | OffsetCommitResponseV8
    | OffsetCommitResponseV9
)



from dataclasses import dataclass
from typing import List

from icestream.kafkaserver.protocol import KafkaRecordBatch


@dataclass
class WALBatch:
    topic: str
    partition: int
    kafka_record_batch: KafkaRecordBatch


@dataclass
class WALFile:
    version: int
    flushed_at: int
    broker_id: str
    batches: List[WALBatch]



from kio.schema.end_quorum_epoch.v0.request import (
    EndQuorumEpochRequest as EndQuorumEpochRequestV0,
)
from kio.schema.end_quorum_epoch.v0.request import (
    RequestHeader as EndQuorumEpochRequestHeaderV0,
)
from kio.schema.end_quorum_epoch.v0.response import (
    EndQuorumEpochResponse as EndQuorumEpochResponseV0,
)
from kio.schema.end_quorum_epoch.v0.response import (
    ResponseHeader as EndQuorumEpochResponseHeaderV0,
)
from kio.schema.end_quorum_epoch.v1.request import (
    EndQuorumEpochRequest as EndQuorumEpochRequestV1,
)
from kio.schema.end_quorum_epoch.v1.request import (
    RequestHeader as EndQuorumEpochRequestHeaderV1,
)
from kio.schema.end_quorum_epoch.v1.response import (
    EndQuorumEpochResponse as EndQuorumEpochResponseV1,
)
from kio.schema.end_quorum_epoch.v1.response import (
    ResponseHeader as EndQuorumEpochResponseHeaderV1,
)

EndQuorumEpochRequestHeader = (
    EndQuorumEpochRequestHeaderV0
    | EndQuorumEpochRequestHeaderV1
)

EndQuorumEpochResponseHeader = (
    EndQuorumEpochResponseHeaderV0
    | EndQuorumEpochResponseHeaderV1
)

EndQuorumEpochRequest = (
    EndQuorumEpochRequestV0
    | EndQuorumEpochRequestV1
)

EndQuorumEpochResponse = (
    EndQuorumEpochResponseV0
    | EndQuorumEpochResponseV1
)



from kio.schema.share_group_describe.v0.request import (
    ShareGroupDescribeRequest as ShareGroupDescribeRequestV0,
)
from kio.schema.share_group_describe.v0.request import (
    RequestHeader as ShareGroupDescribeRequestHeaderV0,
)
from kio.schema.share_group_describe.v0.response import (
    ShareGroupDescribeResponse as ShareGroupDescribeResponseV0,
)
from kio.schema.share_group_describe.v0.response import (
    ResponseHeader as ShareGroupDescribeResponseHeaderV0,
)

ShareGroupDescribeRequestHeader = ShareGroupDescribeRequestHeaderV0

ShareGroupDescribeResponseHeader = ShareGroupDescribeResponseHeaderV0

ShareGroupDescribeRequest = ShareGroupDescribeRequestV0

ShareGroupDescribeResponse = ShareGroupDescribeResponseV0



from kio.schema.describe_delegation_token.v0.request import (
    DescribeDelegationTokenRequest as DescribeDelegationTokenRequestV0,
)
from kio.schema.describe_delegation_token.v0.request import (
    RequestHeader as DescribeDelegationTokenRequestHeaderV0,
)
from kio.schema.describe_delegation_token.v0.response import (
    DescribeDelegationTokenResponse as DescribeDelegationTokenResponseV0,
)
from kio.schema.describe_delegation_token.v0.response import (
    ResponseHeader as DescribeDelegationTokenResponseHeaderV0,
)
from kio.schema.describe_delegation_token.v1.request import (
    DescribeDelegationTokenRequest as DescribeDelegationTokenRequestV1,
)
from kio.schema.describe_delegation_token.v1.request import (
    RequestHeader as DescribeDelegationTokenRequestHeaderV1,
)
from kio.schema.describe_delegation_token.v1.response import (
    DescribeDelegationTokenResponse as DescribeDelegationTokenResponseV1,
)
from kio.schema.describe_delegation_token.v1.response import (
    ResponseHeader as DescribeDelegationTokenResponseHeaderV1,
)
from kio.schema.describe_delegation_token.v2.request import (
    DescribeDelegationTokenRequest as DescribeDelegationTokenRequestV2,
)
from kio.schema.describe_delegation_token.v2.request import (
    RequestHeader as DescribeDelegationTokenRequestHeaderV2,
)
from kio.schema.describe_delegation_token.v2.response import (
    DescribeDelegationTokenResponse as DescribeDelegationTokenResponseV2,
)
from kio.schema.describe_delegation_token.v2.response import (
    ResponseHeader as DescribeDelegationTokenResponseHeaderV2,
)
from kio.schema.describe_delegation_token.v3.request import (
    DescribeDelegationTokenRequest as DescribeDelegationTokenRequestV3,
)
from kio.schema.describe_delegation_token.v3.request import (
    RequestHeader as DescribeDelegationTokenRequestHeaderV3,
)
from kio.schema.describe_delegation_token.v3.response import (
    DescribeDelegationTokenResponse as DescribeDelegationTokenResponseV3,
)
from kio.schema.describe_delegation_token.v3.response import (
    ResponseHeader as DescribeDelegationTokenResponseHeaderV3,
)

DescribeDelegationTokenRequestHeader = (
    DescribeDelegationTokenRequestHeaderV0
    | DescribeDelegationTokenRequestHeaderV1
    | DescribeDelegationTokenRequestHeaderV2
    | DescribeDelegationTokenRequestHeaderV3
)

DescribeDelegationTokenResponseHeader = (
    DescribeDelegationTokenResponseHeaderV0
    | DescribeDelegationTokenResponseHeaderV1
    | DescribeDelegationTokenResponseHeaderV2
    | DescribeDelegationTokenResponseHeaderV3
)

DescribeDelegationTokenRequest = (
    DescribeDelegationTokenRequestV0
    | DescribeDelegationTokenRequestV1
    | DescribeDelegationTokenRequestV2
    | DescribeDelegationTokenRequestV3
)

DescribeDelegationTokenResponse = (
    DescribeDelegationTokenResponseV0
    | DescribeDelegationTokenResponseV1
    | DescribeDelegationTokenResponseV2
    | DescribeDelegationTokenResponseV3
)
